#include-once

; #INDEX# ============================================================================================================
; Title .........: cFileSelectFolder
; AutoIt Version : 3.3.14.5 +
; Language ......: English
; Description ...: Select a file folder from Desktop hierarchy and beyond
; Remarks .......: If your script already has WM_NOTIFY or WM_COMMAND handlers then call the relevant _FSF_WM_####
;                    function from within them, else call _FSF_RegMsg before _cFileleSelectFolder
;                  Thanks to Melba23 for his ChooseFileFolder.au3 and StringSize.au3
; Author ........: c.haslam
; Modified ......; Version 1.6.1
; ====================================================================================================================

#AutoIt3Wrapper_Au3Check_Parameters=-d -w 1 -w 2 -w 3 -w- 4 -w 5 -w 6 -w 7

; #INCLUDES# =========================================================================================================
#include <ButtonConstants.au3>
#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
#include <GuiTreeView.au3>
#include <StaticConstants.au3>
#include <WinAPIConstants.au3>
#include <WindowsConstants.au3>
#include <WinNet.au3>
; ====================================================================================================================

; #GLOBAL VARIABLES, CONSTANTS AND ENUMS# ============================================================================
Global 	$g_cFSF_Expand_Dummy, _		; Branch expansion event
		$g_cFSF_Select_Dummy, _		; Item selection event (checkbox on native treeView)
		$g_cFSF_DblClk_Dummy, _ 	; Expansion or selection
		$g_hFSF_TV, _				; Active TreeView handle
		$g_hFSF_Win, _				; Window
		$g_cLblFSF_Info, _			; Label for reporting user errors in main window
		$g_hFSF_ImageList, _		; imagelist for unselected abd selected treeview items
		$g_sFSF_PreTree, _			; Part of the Windows tree above what's in Treeview: either '' or ends with \
		$g_cInpFSF_NewChild, _		; Input control in DlgNewFolder()
		$g_bFSF_InpChanged, _		; Has user changed input field in DlgNewFOlder() or DlgRenameFolder() ?
		$g_sFSF_EditingMode, _		; 'creating" when in DlgNewFolder(), "editing" when in DlgRenameFOlder
		$g_cInpFSF_NewName, _		; Input control in DlgRename()
		$g_sFSF_UserErrorMessage, _	; User error message (returned when @error=1)
		$g_bFSF_SelectingInitialItem=False, _ ; Avoid updating treeview while _FSF_Expand_Item() is called by _FSF_SelectInitialItem()
		$g_a10FSF_Desktop[0], _	; Desktop tree data
		$g_sFSF_NoFoldersMessage='<No folders meet criteria>'

; For limiting resizing
Global $g_iFSF_GuiMinSizeX,$g_iFSF_GuiMinSizeY,$g_iFSF_GuiMaxSizeX,$g_iFSF_GuiMaxSizeY

; imagelist indices
Global Enum $g_eFSF_IxBlank=0,$g_eFSF_IxFolder,$g_eFSF_IxDT,$g_eFSF_IxLibs,$g_eFSF_IxDocs,$g_eFSF_IxMyDocs,$g_eFSF_IxMusic, _
	$g_eFSF_IxMyMusic,$g_eFSF_IxPics,$g_eFSF_IxMyPics,$g_eFSF_IxVids,$g_eFSF_IxMyVids,$g__IxHomeGp,$g_eFSF_IxUser, _
	$g_eFSF_IxConts,$g_eFSF_IxUDT,$g_eFSF_IxDLs,$g_eFSF_IxFavs,$g_eFSF_IxULinks,$g_eFSF_IxUGames, _
	$g_eFSF_IxUSearches,$g_eFSF_IxUSticky,$g_eFSF_IxSpFolder,$g_eFSF_IxPC,$g_eFSF_IxSysDrive,$g_eFSF_IxCD, _
	$g_eFSF_IxLocalDrive,$g_eFSF_IxMapped,$g_eFSF_IxNetwork,$g_eFSF_IxRemoteDrive

Global Const $g_kFSF_sIllegalChars='\<>:"/|?'

; #CURRENT# ==========================================================================================================
; _cFileSelectFolder	Select a file folder from Desktop hierarchy and beyond
; _FSF_RegMsg			Register WM_#### handlers
; ====================================================================================================================

; #GLOBAL VARIABLE TO BE DECLARED BY CALLING SCRIPT (OPTIONAL)#=======================================================
; $g_bFSF_ReturnUserErrorMessage	To be declared if user errors or argument values to be returned rather than showing in dialog
; $g_bFSF_ShowArguments				To be declared to show values of all arguments
; ====================================================================================================================

; #INTERNAL_USE_ONLY#=================================================================================================
; _FSF_AddtoTree						Add child (and blank item) to tree if meets criteria
; _FSF_CanCreateNewFolder				Can new child folder be added below selected item?
; _FSF_CanRenameOrDeleteFolder			Can selected item be renamed and deleted?
; _FSF_CheckFolderInRoot				Change case from user to actual, error out if non-existent or fails criteriaError out if element of $sRoot does not meet criteria
; _FSF_CheckLineageOfDesktopItemsInRoot		Check for Desktop item not child of another
; _FSF_CheckParameters					Check types of parameters
; _FSF_CollapseAllBlankItems			Collapse all treeview items having blank text
; _FSF_ConnectComputerDrive				Connect to local/mapped drive if not already connected (including CD/DVD)
; _FSF_ConnectMappedDrive				Connect mapped drive per WMI query
; _FSF_CreateDesktopHierarchyData		Create array representing Desktop structure
; _FSF_cWinNet_OpenEnum()				Replacement for _WinNet_OpenEnum() that accepts $RESOURCE* constants
; _FSF_DlgCreateNewFolder				Dialog to create new subfoldere
; _FSF_DlgDeleteFolder					Dialog to delete selected folder
; _FSF_DlgInsertCD						Dialog to request insertion of disc
; _FSF_DlgRenameFolder					Dialog to rename selected folder
; _FSF_DoesFolderContainFolder			Does folder contain folder(s) that meet exclusion, hidden and system properties
; _FSF_EnsureTopOfExpansionIsVisible	Ensure that parent item and as many children as possible are visible
; _FSF_Expand_Item						Expand a treeview item
; _FSF_FillFolder						Add treeview item for folders in a folder
; _FSF_FindChildItem					Get handle of sub-item having a certain text
; _FSF_FindInSubTreeView				Get handle of descandant of an item having a certain text
; _FSF_GetComputerDrives_TextAndIcon	Get text and icon for all local/mapped drives
; _FSF_GetDataVectorForTree				Get hierachy vector for element at bottom of requested tree
; _FSF_GetDataVectorForText				Get hierachy vector for element with requested name
; _FSF_GetDriveLetterForLabel			Get drive letter for drive having requested label
; _FSF_GetDriveMapping					Get mapped drive letters and shares including non-persistent
; _FSF_GetLeftTopForCentringInParent	For centereing a dialog, get left and top given width and height
; _FSF_GetPathForItem					Given handle to treeview item or its tree, get equivalen folder path
; _FSF_GetOnLinePCs						Get names ofcomputers that re online
; _FSF_GetPreTree						Get part of Desktop hierarchy above that in treeview
; _FSF_GetPreTree_Sub					Called by function above
; _FSF_GetShareNamesOnPC				Get share names of a computer
; _FSF_GetTextAndIconForComputerDrive	Get text and icon for la ocal/mapped drive
; _FSF_GUICtrlGetFont					Get the font of a GUI Control
; _FSF_InitializePerDesktop				Initialize treeview for only Desktop items in $sRoot
; _FSF_InitializePerFileSpec			Initialize treeview for only a filepath in $sRoot
; _FSF_InitializePerHybrid				Initialize treeview for Desktop item plus local/mapped folder path or UNC
; _FSF_InitializePerUNC					Initialize treeview for only a UNC in $sRoot
; _FSF_InitializeTreeview				Clean up $sRoot and call one of the InitializePer* functions
; _FSF_IsFolderNameLegal				Does folder name in $sRoot meet Microsoft rules for legality?
; _FSF_IsInVectorBase0					Is value in a zero-based vector?
; _FSF_MergeDriveLetters				Merge 2 vectors of drive letters
; _FSF_ReportUserErroAndCondExit		Report a user error either toa dialog box or for return from _cFileSelectFolder
; _FSF_PointerToStringW					Gets string at a pointer
; _FSF_SelectInitialItem()				Selects item in treeview to be highlighted
; _FSF_SetDesktopItemsInRootToProperCase	Change names from Desktop items from any capitalization
; _FSF_SetImageList						Set up icons for treeview
; _FSF_StandardizeExclFolders			Remove spaces around | and check $sExclFolders
; _FSF_StringSize						Returns size of rectangle required to display string (Melba23)
; _FSF_StringSize_DefaultFontName		Determines Windows default font (Melba23)
; _FSF_StringSize_Error_Close			Releases DC and deleted font object if required after error (Melba23)
; _FSF_VarSizeDialog					Create and polulate variable-size dialog for error messages
; _FSF_WM_COMMAND						WM_COMMAND handler
; _FSF_WM_GETMINMAXINFO					Handler for limiting down-siaing of main dilaog
; _FSF_WM_NOTIFY						WM_NOTIFY handler
; ====================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name ..........: _cFileSelectFolder
; Description ...: Select a file folder from Desktop hierarchy and beyond
; Syntax ........: _cFileSelectFolder([$sTitle = ''[, $sRoot = ''[, $sInitialTree = ''[, $iFolder = 7[, $iWid = 400[,
;                  $iHt = 600[, $iLeft = -1[, $iTop = -1[, $hParent = 0[, $sExclFolders = ''[, $bShowHidden = False[,
;                  $bShowSystem = True]]]]]]]]]]]])
; Parameters ....: $sTitle              - Title of dialog
;                  $sRoot               - Whole or partial Desktop hierarchy, a folder path, a UNC path
;                                         Shows as top item in treeview
;                                         See cFileSlectFolder.pdf for more information
;                  $sInitialTree        - Positon of item in treeview, relative to $sRoot,  to be selected initially
;                                         '' for none to be selected
;                  $iFolder             - folder controls to show: a combination of:
;                                         1 =  New folder, 2 = Delete folder, 4 = Rename folder
;                  $iWid                - Client width of dialog. Negative for resizeable dialog
;                  $iHt                 - Client height of dialog. C resizeable dialog
;                  $iLeft               - Left of dialog. Negative for centered
;                  $iTop                - Top of dialog. Negative for centered
;                  $hParent             - Handle of GUI calling the dialog, (default = 0 - no parent GUI)
;                  $sExclFolders        - Names of folders to exclude, | separated
;                  $bShowHidden         - Show hidden folders
;                  $bShowSystem         - Show system folders
; Return values .: Success:
;                     User clicked on Select: a folder path, e.g. C:\Program Files\AutoIt3
;                     User clicked on Cancel: ''
;                  Failure:
;                     If caller has declared Global $g_bFSF_ReturnUserErrorMessage, it is returned and @error is set to 1
;                     Else dialog shows error message then exits
; Author ........: c.haslam
; Modified ......:
; Remarks .......: *** Call _FSF_RegMsg() before calling this function!!! ***
;                  However if your script has an WM_NOTIFY or _WM_COMMAND handler then do not register the
;                  the handler using _FSF_RegMsg(); rather, call it directly from within your message handler
;
; Note...........: See cFileSelectFolder.pdf, especially for full explanation of $sRoot and $sInitialTree
; ===============================================================================================================================
Func _cFileSelectFolder($sTitle='',$sRoot='',$sInitialTree='',$iFolder=7,$iWid=400,$iHt=600,$iLeft=-1,$iTop=-1, _
	$hParent=0,$sExclFolders='',$bShowHidden=False,$bShowSystem=True)
Local $a10,$hChild,$s
	Local Const $kiResizable = 0x80CC0000 ; BitOR($WS_POPUPWINDOW, $WS_CAPTION, $WS_SIZEBOX)

	If $sRoot=Default Then $sRoot = ''
	If $sRoot='' Then $sRoot = 'Desktop'
	If $iFolder=Default Then $iFolder = 0x07
	If $sInitialTree=Default Then $sInitialTree = ''
	If $bShowHidden=Default Then $bShowHidden = False
	If $bShowSystem=Default Then $bShowSystem = True
	If $sExclFolders=Default Then $sExclFolders = ''
	If $hParent=Default Then $hParent = 0
	If $iWid=Default Then $iWid = 400
	If $iHt=Default Then $iHt = 300
	If $iLeft=Default Then $iLeft =  -1
	If $iTop=Default Then $iTop = -1

	; Clear it in case this is not the first call by the caller
	ReDim $g_a10FSF_Desktop[0]

	Local $bNewFolderControl = BitAND($iFolder,1)=1
	Local $bDeleteFolderControl = BitAND($iFolder,2)=2
	Local $bRenameFolderControl = BitAND($iFolder,4)=4


	If IsDeclared('g_bFSF_ShowArguments') Then
		Local $s1 = ' ('
		If $iFolder=0 Then
			$s1 &= '-no folder controls-'
		Else
			If $bNewFolderControl Then $s1 &= 'New folder,'
			If $bDeleteFolderControl Then $s1 &= 'Delete folder,'
			If $bRenameFolderControl Then $s1 &= 'Rename folder,'
			If BitAND($iFolder,BitNOT(7)) Then $s1 &= '-unknown-,'
			$s1 = StringTrimRight($s1,1)
		EndIf
		$s1 &= ')'
		Local $a20 = [['$sTitle',$sTitle],['$sRoot',$sRoot],['$sInitialTree',$sInitialTree],['$iFolder',$iFolder], _
		['$iWid',$iWid],['$iHt',$iHt],['$iLeft',$iLeft],['$iTop',$iTop],['$hParent',$hParent], _
		['$sExclFolders',$sExclFolders],['$bShowHidden',$bShowHidden],['$bShowSystem',$bShowSystem]]
		$s = ''
		For $i = 0 To UBound($a20,1)-1
			$s &= $a20[$i][0]&' = '&$a20[$i][1]&@CRLF
		Next

		If IsDeclared('g_bFSF_ReturnUserErrorMessage')<>$DECLARED_UNKNOWN Then
			SetError(1)
			Return '_cFileSelectFolder Arguments'&@CRLF&@CRLF&$s
		Else
			_FSF_VarSizeDialog('_cFileSelectFolder: Arguments',$s&@CRLF&@CRLF&'Script will exit',0)
			Exit
		EndIf
	EndIf

	_FSF_CheckParameters($sTitle,$sRoot,$sInitialTree,$iFolder,$iWid,$iHt,$iLeft,$iTop,$hParent, _
		$sExclFolders,$bShowHidden,$bShowSystem)
	If @error Then
		SetError(1)
		Return $g_sFSF_UserErrorMessage
	EndIf

	If $sExclFolders<>'' Then
		; Remove spaces around vertical bars
		$sExclFolders = _FSF_StandardizeExclFolders($sExclFolders)
		If @error Then
			SetError(1)
			Return $g_sFSF_UserErrorMessage
		EndIf
	EndIf
	Local $iStyle =  0x80C80000     ; BitOR($WS_POPUPWINDOW, $WS_CAPTION)	fixed
	Local $iExStyle = 0

	; Set dialog resizing style
	If $iWid < 0  Or $iHt < 0 Then
		$iStyle =  $kiResizable
		; Paints all descendants of a window in bottom-to-top painting order using double-buffering
		$iExStyle = 0x02000000 ; $WS_EX_COMPOSITED
		$iWid = Abs($iWid)
		$iHt = Abs($iHt)
		; Limit resizing
		GUIRegisterMsg($WM_GETMINMAXINFO, '_FSF_WM_GETMINMAXINFO')
		$g_iFSF_GuiMinSizeX = 350
		$g_iFSF_GuiMinSizeY = 300
		$g_iFSF_GuiMaxSizeX = @DesktopWidth
		$g_iFSF_GuiMaxSizeY = @DesktopHeight
	EndIf

	; Check parent
	If $hParent<>0 And Not IsHWnd($hParent) Then
		_FSF_ReportUserErroAndCondExit('In the call to _cFileSelectFolder, $hParent is neither zero nor a window handle')
		SetError(1)
		Return $g_sFSF_UserErrorMessage
	EndIf

	; Check for width and height minima
	If $iWid < 350 Then $iWid = 350
	If $iHt < 300 Then $iHt = 300

	If $hParent<>0 And ($iLeft=-1 Or $iTop=-1) Then
		$a10 = WinGetPos($hParent)
		If $iLeft=-1 Then
			$iLeft = $a10[0] + ($a10[2]/2)-($iWid/2)
		EndIf
		If $iTop=-1 Then
			$iTop = $a10[1] + ($a10[3]-$iHt - 25)/2
		EndIf
	EndIf
	; Set button size
	Local $iButton_Width = Int(($iWid - 50) / 4)
	If $iButton_Width > 80 Then $iButton_Width = 80
	If $iButton_Width < 50 Then $iButton_Width = 50

	; Create GUI window
	$g_hFSF_Win = GUICreate($sTitle, $iWid,$iHt,$iLeft,$iTop,$iStyle,$iExStyle,$hParent)
	GUISetFont(8.5,400,0,'Tahoma',Default,5)
;~ 	GUISetBkColor(0xCECECE)
	Local $cBtnNewFolder,$cBtnDelFolder, $cBtnRenameFolder,$iNextLeft = 10
	If $bNewFolderControl Then
		$cBtnNewFolder = GUICtrlCreateLabel('New folder',10, 10, 54, 17,$SS_NOTIFY)	; frameless button
		$iNextLeft += 54+10
	EndIf
	If $bDeleteFolderControl Then
		$cBtnDelFolder = GUICtrlCreateLabel("Delete folder",$iNextLeft,10,65,17, $SS_NOTIFY)
		$iNextLeft += 65+10
	EndIf
	If $bRenameFolderControl Then
		$cBtnRenameFolder = GUICtrlCreateLabel("Rename folder",$iNextLeft,10,73,17, $SS_NOTIFY)
	EndIf

	; Create TV
	Local $cTreeView = GUICtrlCreateTreeView(10,30,$iWid-20,$iHt-80)
	; Create buttons
	Local $cBtnSelect = GUICtrlCreateButton("Select", $iWid-($iButton_Width+10)*2,$iHt-38,$iButton_Width,30)
	Local $cBtnCancel = GUICtrlCreateButton("Cancel", $iWid -($iButton_Width+10),$iHt-38,$iButton_Width,30)
	$g_cLblFSF_Info = GUICtrlCreateLabel('',20,$iHt-30,200,17)

	; Create dummy control to fire when [+] clicked
	$g_cFSF_Expand_Dummy = GUICtrlCreateDummy()
	; Create dummy control to fire when item checked
	$g_cFSF_Select_Dummy = GUICtrlCreateDummy()
	; Create dummy control to fire on DblClk
	$g_cFSF_DblClk_Dummy = GUICtrlCreateDummy()

	Local $cEsc_Dummy = GUICtrlCreateDummy() ; fires on ESC
	; Set accel keys
	Local $aAccelKeys[1][2] = [["{ESC}", $cEsc_Dummy]]
	GUISetAccelerators($aAccelKeys)

	; Set Global value for handler checks, etc.
	$g_hFSF_TV = GUICtrlGetHandle($cTreeView)

	; Set resizing if required
	If $iStyle = $kiResizable Then
		; Per Help: $GUI_DOCKMENUBAR = $GUI_DOCKHEIGHT + $GUI_DOCKTOP
		;			$GUI_DOCKSTATEBAR = $GUI_DOCKHEIGHT + $GUI_DOCKBOTTOM
		;			$GUI_DOCKBORDERS = $GUI_DOCKLEFT _GUI_DOCKRIGHT + $GUI_DOCKTOP + $GUI_DOCKBOTTOM
		; 3 buttons at top in a row: fixed height
		If $bNewFolderControl Then
			GUICtrlSetResizing($cBtnNewFolder,$GUI_DOCKLEFT + $GUI_DOCKMENUBAR)
		EndIf
		If $bDeleteFolderControl Then
			GUICtrlSetResizing($cBtnDelFolder,$GUI_DOCKLEFT + $GUI_DOCKMENUBAR)
		EndIf
		If $bRenameFolderControl Then
			GUICtrlSetResizing($cBtnRenameFolder,$GUI_DOCKLEFT + $GUI_DOCKMENUBAR)
		EndIf
		; top of treeview doesn't change, bottom does
		GUICtrlSetResizing($cTreeView, $GUI_DOCKBORDERS)
		; 2 buttons and label in a row. Vertical space for them doesn't change
		GUICtrlSetResizing($g_cLblFSF_Info,$GUI_DOCKWIDTH + $GUI_DOCKSTATEBAR)
		GUICtrlSetResizing($cBtnCancel,  $GUI_DOCKWIDTH +   $GUI_DOCKSTATEBAR)
		GUICtrlSetResizing($cBtnSelect,  $GUI_DOCKWIDTH +   $GUI_DOCKSTATEBAR)
	EndIf
	GUISetState(@SW_SHOW,$g_hFSF_Win)

;~ 	Local $hParentWindow = _WinAPI_GetParent($g_hFSF_Win)
;~ 	WinSetState($hParentWindow,'',@SW_DISABLE)

	_GUICtrlTreeView_Add($cTreeView,0,'Loading ...')

	_FSF_InitializeTreeview($g_hFSF_TV,$sRoot,$sExclFolders,$bShowHidden,$bShowSystem)
	If @error Then
		SetError(1)
		Return $g_sFSF_UserErrorMessage
	EndIf

	If $sInitialTree<>'' Then
		_FSF_SelectInitialItem($sInitialTree,$sRoot,$g_hFSF_TV,$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return $g_sFSF_UserErrorMessage
		EndIf
	EndIf
	_GUICtrlTreeView_EndUpdate($g_hFSF_TV)

	; Change to MessageLoop mode
	Local $nOldOpt = Opt('GUIOnEventMode', 0)
	Local $hItem,$sPath,$sText,$sTree

	While 1
		Switch GUIGetMsg()
		Case $cBtnCancel,$cEsc_Dummy,$GUI_EVENT_CLOSE
			GUIDelete($g_hFSF_Win)
			; Restore previous mode
			Opt('GUIOnEventMode', $nOldOpt)
			Return ''
		Case $cBtnSelect
			$hItem = _GUICtrlTreeView_GetSelection($g_hFSF_TV)
			If $hItem=0 Then
				GUICtrlSetData($g_cLblFSF_Info,'Select an item')
				Sleep(3000)
				GUICtrlSetData($g_cLblFSF_Info,'')
			Else
				Local $sSelectedPath = _FSF_GetPathForItem($hItem)
				If $sSelectedPath<>'' Then
					GUIDelete($g_hFSF_Win)
					_GUIImageList_Destroy($g_hFSF_ImageList)
					; Restore previous mode
					Opt('GUIOnEventMode', $nOldOpt)
					Return $sSelectedPath&'\'
				Else
					GUICtrlSetData($g_cLblFSF_Info,'Invalid selection')
					Sleep(3000)
					GUICtrlSetData($g_cLblFSF_Info,'')
				EndIf
			EndIf
		Case $cBtnNewFolder
			$hItem =_GUICtrlTreeView_GetSelection($g_hFSF_TV)
			If Not _FSF_CanCreateNewFolder($hItem) Then
				GUICtrlSetData($g_cLblFSF_Info,'Can''t insert a subfolder here')
				Sleep(3000)
				GUICtrlSetData($g_cLblFSF_Info,'')
				ContinueLoop
			EndIf
			$sTree = StringReplace(_GUICtrlTreeView_GetTree($g_hFSF_TV),'|','\')	; all
			$sText = _FSF_DlgCreateNewFolder($hItem,$sTree,$sExclFolders)
			If $sText<>'' Then
				If _GUICtrlTreeView_GetChildCount($g_hFSF_TV,$hItem)=1 Then
					$hChild = _GUICtrlTreeView_GetFirstChild($g_hFSF_TV,$hItem)
					If  _GUICtrlTreeView_GetText($g_hFSF_TV,$hChild)=$g_sFSF_NoFoldersMessage Then
						; Delete no folders message
						_GUICtrlTreeView_Delete($g_hFSF_TV,$hChild)
					EndIf
				EndIf

				$hChild = _GUICtrlTreeView_AddChild($g_hFSF_TV,$hItem,$sText,$g_eFSF_IxFolder,$g_eFSF_IxFolder)
				_GUICtrlTreeView_Expand($g_hFSF_TV,$hItem)
				GUICtrlSendToDummy($g_cFSF_Expand_Dummy,0)	; de-select
				_SendMessage($g_hFSF_TV, $TVM_SORTCHILDREN, 0, $hItem, 0, "wparam", "handle")
				_FSF_CollapseAllBlankItems($g_hFSF_TV)
				$sPath = _FSF_GetPathForItem($hChild)
				DirCreate($sPath)
			EndIf
		Case $cBtnRenameFolder
			$hItem =_GUICtrlTreeView_GetSelection($g_hFSF_TV)
			If Not _FSF_CanRenameOrDeleteFolder($hItem) Then
				GUICtrlSetData($g_cLblFSF_Info,'Can''t rename this item')
				Sleep(3000)
				GUICtrlSetData($g_cLblFSF_Info,'')
				ContinueLoop
			EndIf
			$sPath = StringReplace(_GUICtrlTreeView_GetTree($g_hFSF_TV,$hItem),'|','\')	; all
			Local $hItemParent = _GUICtrlTreeView_GetParentHandle($g_hFSF_TV,$hItem)
			Local $sPathParent = StringReplace(_GUICtrlTreeView_GetTree($g_hFSF_TV,$hItemParent),'|','\')	; all
			Local $sCurName = _GUICtrlTreeView_GetText($g_hFSF_TV,$hItem)
			$sText = _FSF_DlgRenameFolder($hItem,$sPathParent,$sCurName,$sExclFolders)
			If $sText<>'' Then
				$sPath = _FSF_GetPathForItem($hItem)	; without drive name, e.g. F:\AutoIt scripts
				_GUICtrlTreeView_SetText($g_hFSF_TV,$hItem,$sText)
				Local $sNuPath = _FSF_GetPathForItem($hItem)
				DirMove($sPath,$sNuPath)
			EndIf
		Case $cBtnDelFolder
			$hItem =_GUICtrlTreeView_GetSelection($g_hFSF_TV)
			If Not _FSF_CanRenameOrDeleteFolder($hItem) Then
				GUICtrlSetData($g_cLblFSF_Info,'Can''t delete this item')
				Sleep(3000)
				GUICtrlSetData($g_cLblFSF_Info,'')
				ContinueLoop
			EndIf
			$sTree = StringReplace(_GUICtrlTreeView_GetTree($g_hFSF_TV),'|','\')	; all
			Local $sAction = _FSF_DlgDeleteFolder($hItem,$sTree)
			If $sAction='' Then	; if cancelled
				ContinueLoop
			EndIf
			$sPath = _FSF_GetPathForItem($hItem)
			If $sAction='r' Then	; recycle
				If FileRecycle($sPath) Then
					_FSF_VarSizeDialog('Folder recycled', _
						$sPath&@CRLF&@CRLF&'has been sent to the Recycle Bin',3)
				Else
					_FSF_VarSizeDialog('Folder in use', _
						$sPath&@CRLF&@CRLF&' is in use by another application',0)	; no timeout
					ContinueLoop
				EndIf
			Else	; permanent deletion
				If DirRemove($sPath,1) Then	; delete subdirs too
					_FSF_VarSizeDialog('Folder deleted', _
						$sPath&@CRLF&@CRLF&'has been permanently deleted',3)
				Else
					_FSF_VarSizeDialog('Deleting folder failed', _
					$sPath&@CRLF&@CRLF&'cannot be deleted.'&@CRLF&@CRLF& 'It may be in use',0)
					ContinueLoop
				EndIf
			EndIf

			; Replace the folder item with the no-folders message if parent is top item
			$hParent = _GUICtrlTreeView_GetParentHandle($g_hFSF_TV,$hItem)
			_GUICtrlTreeView_Delete($g_hFSF_TV,$hItem)
			If _GUICtrlTreeView_IsFirstItem($g_hFSF_TV,$hParent) Then
				If _GUICtrlTreeView_GetChildCount($g_hFSF_TV,$hParent)=-1 Then	; none!
					_GUICtrlTreeView_AddChild($g_hFSF_TV,$hParent,$g_sFSF_NoFoldersMessage,$g_eFSF_IxBlank,$g_sFSF_NoFoldersMessage)
				EndIf
			EndIf

		Case $g_cFSF_Expand_Dummy
			; Get expanded item hamdle
			$hItem = GUICtrlRead($g_cFSF_Expand_Dummy)
			If $hItem Then
				_FSF_Expand_Item($hItem,$g_hFSF_TV,$sExclFolders,$bShowHidden,$bShowSystem)
				If @error Then
					SetError(1)
					Return $g_sFSF_UserErrorMessage
				EndIf
			EndIf
			GUICtrlSendToDummy($g_cFSF_Expand_Dummy, 0)
  		EndSwitch
	WEnd
EndFunc

Func _FSF_SelectInitialItem($sInitialTree,$sRoot,$hTV,$sExclFolders,$bShowHidden,$bShowSystem)
Local $hChild,$s

	$g_bFSF_SelectingInitialItem = True

	Local $a10InitTree[0]
	If StringInStr($sInitialTree,'\')=0 Then
		ReDim $a10InitTree[1]
		$a10InitTree[0] = $sInitialTree
	Else
		$a10InitTree = StringSplit($sInitialTree,'\',0+2)
	EndIf

	; Find first or only InitialTree element in descendants
	Local $hTop = _GUICtrlTreeView_GetItemHandle($hTV,0)
	Local $hInitItem =  _FSF_FindInSubTreeView($hTV,$hTop,$a10InitTree[0])

	If $hInitItem<>0 Then
		 _GUICtrlTreeView_SelectItem($hTV,$hInitItem)
	Else

		Local $sMsg = '$sInitialTree: "'&$a10InitTree[0]&'" not found in treeview.'

		If IsDeclared('g_bFSF_ReturnUserErrorMessage')<>$DECLARED_UNKNOWN Then
			$g_sFSF_UserErrorMessage = '_cFileSelectFolder User error'&@CRLF&$sMsg&@CRLF& _
			'Call _cFileSelectFolder with $sInitialTree set to "" to see possible values for the first element'&@CRLF& _
			'It can be the text of any item below the top item'
			SetError(1)
			Return
		Else
			_GUICtrlTreeView_EndUpdate($hTV)
			_FSF_ReportUserErroAndCondExit($sMsg&@CRLF&@CRLF& _
				'The first element must be the text of an item shown below the top item')
			Exit
		EndIf
	EndIf

	; Find children in rest of $sInitialTree
	Local $hParent = $hInitItem
	_GUICtrlTreeView_BeginUpdate($hTV)
	For $i = 1 To UBound($a10InitTree)-1
		_GUICtrlTreeView_SelectItem($hTV,$hParent)
		_FSF_Expand_Item($hParent,$hTV,$sExclFolders,$bShowHidden,$bShowSystem)
		$hChild = _FSF_FindChildItem($hTV,$hParent,$a10InitTree[$i])
		If $hChild=0 Then
			_GUICtrlTreeView_EndUpdate($hTV)
			If $i=0 Then
				$s = $sRoot
			Else
				$s = $a10InitTree[$i-1]
			EndIf
			_FSF_ReportUserErroAndCondExit('"'&$a10InitTree[$i]&'" is not a child of "'&$s&'"')
			SetError(1)
			Return
		EndIf
		$hParent = $hChild
	Next
	_GUICtrlTreeView_SelectItem($hTV,$hParent)
	_FSF_Expand_Item($hParent,$hTV,$sExclFolders,$bShowHidden,$bShowSystem)

	; Collapse all other items at level of selected
	Local $hGrandParent = _GUICtrlTreeView_GetParentHandle($hTV,$hParent)

	Local $iCount = _GUICtrlTreeView_GetChildCount($hTV,$hGrandParent)
	For $i = 0 To $iCount-1
		If $i = 0 Then
			$hChild = _GUICtrlTreeView_GetFirstChild($hTV,$hGrandParent)
		Else
			$hChild = _GUICtrlTreeView_GetNextChild($hTV,$hChild)
		EndIf
		If $hChild<>$hParent Then
			_GUICtrlTreeView_Expand($hTV,$hChild,False)
		EndIf
	Next

	_GUICtrlTreeView_EnsureVisible($hTV,$hTop)
	_GUICtrlTreeView_EnsureVisible($hTV,$hParent)

	_GUICtrlTreeView_EndUpdate($hTV)

	$g_bFSF_SelectingInitialItem = False
EndFunc

Func _FSF_FindInSubTreeView($hTV,$hParent,$sText)
Local $hChild,$hitem,$hRet
	Local $iCount = _GUICtrlTreeView_GetChildCount($hTV,$hParent)
	If $iCount=-1 Then
		Return 0
	Else
		For $i = 0 To $iCount-1
			If $i = 0 Then
				$hChild = _GUICtrlTreeView_GetFirstChild($hTV,$hParent)
			Else
				$hChild = _GUICtrlTreeView_GetNextSibling($hTV,$hChild)
			EndIf
			If _GUICtrlTreeView_GetText($hTV,$hChild)=$sText Then
				$hRet = $hChild
				If $hRet<>0 Then
					Return $hRet
				EndIf
			Else
				$hitem = _FSF_FindInSubTreeView($hTV,$hChild,$sText)
				$hRet = $hitem
				If $hRet<>0 Then
					Return $hRet
				EndIf

			EndIf
		Next
	EndIf
	Return $hRet
EndFunc

Func _FSF_CheckParameters($sTitle,$sRoot,$sInitialTree,$iFolder,$iWid,$iHt,$iLeft,$iTop,$hParent, _
	$sExclFolders,$bShowHidden,$bShowSystem)

	Local $a20 = [['$sTitle',$sTitle],['$sRoot',$sRoot],['$sInitialTree',$sInitialTree],['$sExclFolders',$sExclFolders]]
	For $i = 0 To UBound($a20,1)-1
		If Not IsString($a20[$i][1]) Then
			_FSF_ReportUserErroAndCondExit($a20[$i][0]&' "'&$a20[$i][1]&'" must be a string')
			SetError(1)
			Return
		EndIf
	Next

	Local $a20 = [['$iFolder',$iFolder],['$iWid',$iWid],['$iHt',$iHt],['$iLeft',$iLeft],['$iTop',$iTop]]
	For $i = 0 To UBound($a20,1)-1
		If Not _FSF_IsNumeric($a20[$i][1]) Then
			_FSF_ReportUserErroAndCondExit($a20[$i][0]&' "'&$a20[$i][1]&'" must be a number')
			SetError(1)
			Return
		EndIf
	Next

	If Not (IsHWnd($hParent) Or $hParent=0) Then
		_FSF_ReportUserErroAndCondExit('$hParent "'&$a20[$i][1]&'" must be 0 or a window handle')
		SetError(1)
		Return
	EndIf

	If BitAND($iFolder,BitNOT(7)) Then
		_FSF_ReportUserErroAndCondExit('$iFolder: Unknown bit combination 0x'&Hex($iFolder)&' ('&$iFolder&')')
		SetError(1)
		Return
	EndIf

	Local $a20 = [['$bShowHidden',$bShowHidden],['$bShowSystem',$bShowSystem]]
	For $i = 0 To UBound($a20,1)-1
		If Not IsBool($a20[$i][1]) Then
			_FSF_ReportUserErroAndCondExit($a20[$i][0]&' "'&$a20[$i][1]&'" must be boolean')
			SetError(1)
			Return
		EndIf
	Next
EndFunc

#CS
; For diagnostic purposes only
Func gofi($nln,$sVar,$hItem)
	If $hItem=0 Then
		_guidebug($nln,$svar, 0)
	Else
		_guidebug($nln,$svar,_GUICtrlTreeView_GetText($g_hFSF_TV,$hItem)&'  0x'&$hitem)
	EndIf
EndFunc

; For diagnostic purposes only
Func cofi($nln,$sVar,$hItem)
	If $hItem=0 Then
		_consdebug($nln,$svar, 0)
	Else
		_consdebug($nln,$svar,_GUICtrlTreeView_GetText($g_hFSF_TV,$hItem)&'  0x'&$hitem)
	EndIf
EndFunc
#CE

Func _FSF_IsNumeric($vN)
	Return IsNumber($vN) Or StringIsInt($vN) Or StringIsFloat($vN)
EndFunc

; Return 0 if not child of parent
Func _FSF_FindChildItem($hTV,$hParent,$sText)
	Local $s,$hRet=0
	Local $iCount = _GUICtrlTreeView_GetChildCount($hTV,$hParent)
	If $iCount<>-1 Then
		Local $hChild = _GUICtrlTreeView_GetFirstChild($hTV,$hParent)
		$s = _GUICtrlTreeView_GetText($hTV,$hChild)
		If $s=$sText Then
			$hRet = $hChild
		Else
			For $i = 2 To $iCount
				$hChild = _GUICtrlTreeView_GetNextSibling($hTV,$hChild)
				$s = _GUICtrlTreeView_GetText($hTV,$hChild)
				If $s=$sText Then
					$hRet = $hChild
					ExitLoop
				EndIf
			Next
		EndIf
	EndIf
	Return $hRet
EndFunc

Func _FSF_CanCreateNewFolder($hItem)
	; If no item seleceted
	If $hItem=0 Then
		Return False
	EndIf

	; If Desktop item has no equivalent
	If _FSF_GetPathForItem($hItem)='' Then
		Return False
	EndIf
	Return True
EndFunc

Func _FSF_CanRenameOrDeleteFolder($hItem)
	; If no item selected
	If $hItem=0 Then
		Return False
	EndIf

	If $hItem=_GUICtrlTreeView_GetFirstItem($g_hFSF_TV) Then
		Return False
	EndIf

	; If Desktop item is above terminal, including Computer, Network and Network\<computer>
	If _FSF_GetPathForItem($hItem)='' Then
		Return False
	EndIf

	; If a terminal Desktop item
	Local $hParent = _GUICtrlTreeView_GetParentHandle($g_hFSF_TV,$hItem)
	If _FSF_GetPathForItem($hParent)='' Then
		Return False
	EndIf

	Local $sParent = _GUICtrlTreeView_GetText($g_hFSF_TV,$hParent)
	If $sParent='Users' Then
		Return False
	EndIf

	Local $sTree = $g_sFSF_PreTree&StringReplace(_GUICtrlTreeView_GetTree($g_hFSF_TV,$hParent),'|','\')
	; If renaming or deleting a local/mapped drive or a computer
	If $sTree='Desktop\Computer' Or $sTree='Desktop\Network' Then
		Return False
	EndIf

	Local $hGrandParent = _GUICtrlTreeView_GetParentHandle($g_hFSF_TV,$hParent)
	Local $sGrandParent = _GUICtrlTreeView_GetText($g_hFSF_TV,$hGrandParent)
	; If a computer
	If $sGrandParent='Desktop' And $sParent='Network' Then
		Return False
	EndIf

	; If a drive or Users
	If $sGrandParent='Network' Then
		Return False
	EndIf

	; If e.g. Users\Chris\Desktop
	If $sGrandParent='Users' Then
		Return False
	EndIf

	Return True
EndFunc

Func _FSF_StandardizeExclFolders($sExclFolders)
Local $a10,$s
	; Remove spaces around vertical bars
	Local $a10Excl = StringSplit($sExclFolders,'|',0+2)		; char,0-based
	$sExclFolders = ''
	For $i = 0 To UBound($a10Excl)-1
		$sExclFolders &= '|'&StringStripWS($a10Excl[$i],1+2)	; left. right
	Next
	$sExclFolders = StringMid($sExclFolders,2)

	$a10Excl = StringSplit($sExclFolders,'|',0+2)		; char,0-based
	For $i = 0 To UBound($a10Excl)-1
		If $a10Excl[$i]='' Then
			If UBound($a10Excl)=1 Then
				$s = 'Name'
			ElseIf $i=0 Then
				$s = 'First name'
			ElseIf $i=UBound($a10Excl)-1 Then
				$s = 'Last name'
			Else
				$s = 'Name '&($i+1)
			EndIf
			_FSF_ReportUserErroAndCondExit('$sExclFolders: '&$s&' is missing')
			SetError(1)
			Return
		EndIf
		$a10 = StringRegExp($a10Excl[$i],'['&$g_kFSF_sIllegalChars&']',1)
		If @error=0 Then
			_FSF_ReportUserErroAndCondExit('$sExclFolders: Illegal character '&$a10[0])
			SetError(1)
			Return
		EndIf
	Next
	Return $sExclFolders
EndFunc

Func _FSF_SetImageList($hTV)

	; 32-bit colour, use a mask, images in list are mirrored, 31 images initially
	$g_hFSF_ImageList = _GUIImageList_Create(16, 16, 5, 3, 31)
	_GUIImageList_Add($g_hFSF_ImageList, _GUICtrlTreeView_CreateSolidBitMap($hTV,0xffffFF,16,16))	; white so blank
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',3)	; folder
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',105)	; Desktop
	_GUIImageList_AddIcon($g_hFSF_ImageList,@WindowsDir&'\explorer.exe',0)	; Libraries
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',14)	; Documents
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',107)	; My Documents
	_GUIImageList_AddIcon($g_hFSF_ImageList,'wmploc.dll',31)	; Music
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',103)	; My Music
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',67)	; Pictures
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',108)	; My Pictures
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',18)	; Videos
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',178)	; My Videos
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',18)	; Homegroup
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',117)	; User
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',172)	; User Contacts
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',174)	; User Desktop
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',175)	; User Downloads
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',199)	; User Favorites
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',176)	; User Links
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',177)	; Saved Games
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',13)	; Searches
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',204)	; Sticky
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',165)	; SpecialFolder
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',15)	; PC
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',31)	; sysDrive
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',11)	; CD drive
	_GUIImageList_AddIcon($g_hFSF_ImageList,'imageres.dll',27)	; local drive
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',275)	; mapped
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',17)	; network
	_GUIImageList_AddIcon($g_hFSF_ImageList,'shell32.dll',273)	; remote drive
	_GUICtrlTreeView_SetNormalImageList($hTV, $g_hFSF_ImageList)
EndFunc

Func _FSF_InitializeTreeview($hTV,$sRoot,$sExclFolders,$bShowHidden,$bShowSystem)
Local $iPos,$a10sRoot

	If $sRoot<>'' Then
		; Note UNC path so can be restored below
		Local $bSlashSlash = StringLeft($sRoot,2)='\\'
		Local $iStart = $bSlashSlash ? 2 : 0

		If StringInStr($sRoot,' ') Then
			; Remove spaces around backslashes
			$a10sRoot = StringSplit($sRoot,'\',0+2)	; char,0-based
			If $bSlashSlash And UBound($a10sRoot)<=2 Then
				_FSF_ReportUserErroAndCondExit('$sRoot="\\" is invalid')
				SetError(1)
				Return
			EndIf
			$sRoot = ''
			For $i = 0 To UBound($a10sRoot)-1
				$sRoot &= '\'&StringStripWS($a10sRoot[$i],1+2)	; left,right
			Next
			$sRoot = StringMid($sRoot,2)
		EndIf

		$a10sRoot = StringSplit($sRoot,'\',0+2)	; char,0-based
		For $i = $iStart To UBound($a10sRoot)-1
			If $a10sRoot[$i]='' Then
				If $i=$iStart Then
					_FSF_ReportUserErroAndCondExit('$sRoot: First element is missing')
					SetError(1)
					Return
				ElseIf $i<UBound($a10sRoot)-1 Then
					_FSF_ReportUserErroAndCondExit('$sRoot: Element '&($i-$iStart+1)&' is missing')
					SetError(1)
					Return
				EndIf
			EndIf
		Next
	EndIf

	_FSF_SetImageList($hTV)

	; Convert drive label to drive letter
	If StringInStr($sRoot,':') Then
		Local $sDriveLabel=''
		$a10sRoot = StringSplit($sRoot,'\',0+2)
		For $i = 0 To UBound($a10sRoot)-1
			If StringRight($a10sRoot[$i],1)=':' Then
				$iPos = StringInStr($a10sRoot[$i],':')
				If $iPos>2 Then
					$sDriveLabel = StringLeft($a10sRoot[$i],$iPos-1)
					ExitLoop
				EndIf
			EndIf
		Next
		Local $sDriveLetter
		If $sDriveLabel<>'' Then
			$sDriveLetter = _FSF_GetDriveLetterForLabel($sDriveLabel)
			If $sDriveLetter='' Then
				_FSF_ReportUserErroAndCondExit('$sRoot: There is no drive labelled "'&$sDriveLabel&'"')
				SetError(1)
				Return
		EndIf
			$sRoot = StringReplace($sRoot,$sDriveLabel&':',$sDriveLetter)
		EndIf
	EndIf

	; If local or mapped drive
	If StringRegExp($sRoot,'^[a-zA-Z]:') Then
		_FSF_InitializePerFileSpec($hTV,$sRoot,$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return
		EndIf
	; If UNC
	ElseIf StringLeft($sRoot,2)='\\' Then
		_FSF_InitializePerUNC($hTV,$sRoot,$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return
		EndIf
	Else
		Local  $a10DesktopWithNetwork,$iEndDT
		$sRoot = _FSF_SetDesktopItemsInRootToProperCase($sRoot,  $a10sRoot,  $iendDT,  $a10DesktopWithNetwork, _
			$sExclFolders)
		If @error Then
			SetError(1)
			Return
		EndIf
		If $iEndDT>0 Then
			_FSF_CheckLineageOfDesktopItemsInRoot($a10sRoot,$iEndDT,$a10DesktopWithNetwork)
			If @error Then
				SetError(1)
				Return
			EndIf
		EndIf
		Local $b = False
		For $i = 0 To $iendDT
			If $a10sRoot[$i]='Network' Then
				$b = True
				ExitLoop
			EndIf
		Next
		Local $a10Desktop
		If  $b Then
			$a10Desktop = $a10DesktopWithNetwork
		Else
			$a10Desktop = _FSF_CreateDesktopHierarchyData(False,$sExclFolders)
		EndIf

		If $iEndDT=UBound($a10sRoot)-1 Then
			_FSF_InitializePerDesktop($hTV,$sRoot,$a10Desktop,$sExclFolders,$bShowHidden,$bShowSystem)
		Else
			_FSF_InitializePerHybrid($hTV,$sRoot,$a10Desktop,$iEndDT,$sExclFolders,$bShowHidden,$bShowSystem)
		EndIf
		If @error Then
			SetError(1)
			Return
		EndIf
	EndIf
	_GUICtrlTreeView_EnsureVisible($hTV,0)
EndFunc

; @error=1 => fatal error
; $iEndDT => index of last Desktop item
Func _FSF_SetDesktopItemsInRootToProperCase($sRoot, ByRef $a10sRoot, ByRef $iendDT, ByRef $a10DesktopWithNetwork, _
	$sExclFolders)
	$a10DesktopWithNetwork = _FSF_CreateDesktopHierarchyData(True,$sExclFolders)

	; Get names of items in Desktop
	Local $sDTnames,$a10d1,$a10d2,$a10d3
	$sDTnames = '|'&$a10DesktopWithNetwork[0]
	For $i1 = 3 To UBound($a10DesktopWithNetwork)-1
		$a10d1 = $a10DesktopWithNetwork[$i1]
		If StringInStr($sDTnames,'|'&$a10d1[0]&'|')=0 Then
			$sDTnames &= '|'&$a10d1[0]
		EndIf

		For $i2 = 3 To UBound($a10d1)-1
			$a10d2 = $a10d1[$i2]
			If StringInStr($sDTnames,'|'&$a10d2[0]&'|')=0 Then
				$sDTnames &= '|'&$a10d2[0]
			EndIf
			For $i3 = 3 To UBound($a10d2)-1
				$a10d3 = $a10d2[$i3]
				If StringInStr($sDTnames,'|'&$a10d3[0]&'|')=0 Then
					$sDTnames &= '|'&$a10d3[0]
				EndIf
			Next
		Next
	Next

	; Look for Desktop items and ensure that they have proper case
	$a10sRoot = StringSplit($sRoot,'\',0+2)
	If @error Then	; If only 1 element
		Local $a10 = [$sRoot]
		$a10sRoot = $a10
	EndIf
	Local $bChanged=False
	$iEndDT = UBound($a10sRoot)-1
	For $i = 0 To UBound($a10sRoot)-1
		Local $iPst = StringInStr('|'&$sDTnames&'|','|'&$a10sRoot[$i]&'|',0,1)	; case_insens, first
		If $iPst=0 Or @error Then
			$iEndDT = $i-1
			ExitLoop
		Else
			Local $iPend = StringInStr('|'&$sDTnames&'|','|',0,1,$iPst+1)
			Local $s = StringMid($sDTnames,$ipst,$iPend-$iPst-1)
			If Not ($a10sRoot[$i]==$s) Then	; case sensitive
				$a10sRoot[$i] = $s
				$bChanged = True
			EndIf
		EndIf
	Next

	If $iEndDT=-1 Then
		_FSF_ReportUserErroAndCondExit('$sRoot: "'&$sRoot&'" is invalid'&@CRLF&@CRLF& _
		'Values of $sRoot:'&@CRLF&@CRLF& _
		'To show the whole Desktop hierarchy, use "" or "Desktop"'&@CRLF&@CRLF& _
		'To show a branch of the Desktop hierachy, use one item or <item1>\<item2>...'&@CRLF&@CRLF& _
		'If the last item does not have an equivalent path to a folder, shows items below it that do'&@CRLF&@CRLF& _
		'To show a path to a local or mapped folder, use its path (starting with drive letter)'&@CRLF&@CRLF& _
		'A drive label followed by a colon may be used in place of a drive letter'&@CRLF&@CRLF& _
		'To show a path to an unmapped remote folder, use its UNC path'&@CRLF&@CRLF& _
		'See cFileSlectFolder.pdf for more information. Example scripts are included in the .zip file'&@CRLF&@CRLF&@CRLF& _
		'When Select is clicked, cFileSelectFolder returns the path to the selected folder'&@CRLF&@CRLF& _
		'When Cancel is clicked, it returns ""'&@CRLF&@CRLF&@CRLF& _
		'The default is for errors to show in a dialog'&@CRLF&@CRLF& _
		'If you code Global $g_bFSF_ReturnUserErrorMessage before calling this function, error messages'&@CRLF& _
		'are returned and @error is set to 1')
		SetError(1)
		Return
	EndIf
	If $bChanged Then
		$sRoot = $a10sRoot[0]
		For $i = 1 To UBound($a10sRoot)-1
			$sRoot &= '\'&$a10sRoot[$i]
		Next
	EndIf
	Return $sRoot
EndFunc

Func _FSF_CheckLineageOfDesktopItemsInRoot($a10sRoot,$iEndDT,$a10DesktopWithNetwork)
Local $a10data
	Local $a10Parent = _FSF_GetDataVectorForText($a10DesktopWithNetwork,$a10sRoot[0])
	For $i = 0 To $iendDT-1
		Local $bOK = False
		For $iJ = 3 To UBound($a10Parent)-1
			If ($a10Parent[$iJ])[0]=$a10sRoot[$i+1] Then
				$a10data = $a10Parent[$iJ]
				$bOK = True
				ExitLoop
			EndIf
		Next
		If Not $bOK Then
			_FSF_ReportUserErroAndCondExit('$sRoot: "'&$a10sRoot[$i+1]&'" is not a child of "'&$a10Parent[0]&'"')
			SetError(1)
			Return
		EndIf
		$a10Parent = $a10data
	Next
EndFunc

Func _FSF_InitializePerDesktop($hTV,$sRoot,$a10Desktop,$sExclFolders,$bShowHidden,$bShowSystem)
Local $h,$hPC,$a10Data[3],$hItem
	If $sRoot='Network' Or $sRoot='Desktop\Network' Then
		$a10Data[0] ='Network'
		$a10Data[1] = $g_eFSF_IxNetwork
		$a10Data[2] = 'b'				; indicate that it can be expanded

		Local $a10OnlinePCs = _FSF_GetOnLinePCs()

		Local $iQ = UBound($a10OnlinePCs)
		If $iQ=0 Then
			_FSF_ReportUserErroAndCondExit('$sRoot is "'&$sRoot&'" but there is no network')
		ElseIf $iQ=1 Then
			_FSF_ReportUserErroAndCondExit('$sRoot is "'&$sRoot&'" but there are no other computers on the network')
			SetError(1)
			Return
		EndIf

		; There are computers on the network other than this one
		_GUICtrlTreeView_DeleteAll($hTV)
		$hItem = _GUICtrlTreeView_Add($hTV,0,$sRoot,$g_eFSF_IxNetwork,$g_eFSF_IxNetwork)
		_GUICtrlTreeView_SetBold($hTV,$hItem)
		Local $a10Shares[UBound($a10OnlinePCs)]
		GUICtrlSetData($g_cLblFSF_Info,'Searching for shares')
		For $i = 0 To UBound($a10OnlinePCs)-1
			$a10Shares[$i] = _FSF_GetShareNamesOnPC($a10OnlinePCs[$i])
		Next
		GUICtrlSetData($g_cLblFSF_Info,'')

		_GUICtrlTreeView_BeginUpdate($hTV)
		For $i = 0 To UBound($a10OnlinePCs)-1
			; Add computers and their shares to treeview
			$hPC = _GUICtrlTreeView_AddChild($hTV,$hItem,$a10OnlinePCs[$i],$g_eFSF_IxPC,$g_eFSF_IxPC)
			For $iJ = 0 To UBound($a10Shares[$i])-1
				$h = _GUICtrlTreeView_AddChild($hTV,$hPC,($a10Shares[$i])[$iJ],$g_eFSF_IxRemoteDrive, _
					$g_eFSF_IxRemoteDrive)
				_GUICtrlTreeView_AddChild($hTV,$h,'',$g_eFSF_IxBlank,$g_eFSF_IxBlank)
			Next
		Next
		_GUICtrlTreeView_Expand($hTV,$hItem)
		GUICtrlSendToDummy($g_cFSF_Expand_Dummy,0)	; de-select last item
		_FSF_CollapseAllBlankItems($hTV)
		If $sRoot='Network' Then
			$g_sFSF_PreTree = 'Desktop\'
		Else
			$g_sFSF_PreTree = ''
		EndIf
	Else	; if $sRoot is 'Dekstop'
		$a10Data = _FSF_GetDataVectorforTree($a10Desktop,$sRoot)
		Local $a10sRoot = StringSplit($sRoot,'\',0+2)
		$g_sFSF_PreTree =_FSF_GetPreTree($a10Desktop,$a10sRoot[0])

		; Remove Network if not conected to router
		If Not _FSF_IsAnyPcOnline() Then
			ReDim $a10Data[UBound($a10Data)-1]
		EndIf

		_GUICtrlTreeView_BeginUpdate($hTV)

		_GUICtrlTreeView_DeleteAll($hTV)	; Get rid of "Loading ..."
		$hItem = _GUICtrlTreeView_Add($hTV,0,$sRoot,$a10Data[1],$a10Data[1])
		_GUICtrlTreeView_SetBold($hTV,$hItem)
		Local $hItemTop = $hItem
		If UBound($a10Data)=3 Then	; if terminal, i.e. no Desktop children
			_GUICtrlTreeView_AddChild($hTV,$hItem,'',$g_eFSF_IxBlank)
			Local $sPath = _FSF_GetPathForItem($hItem)
			If Not _FSF_DoesFolderContainFolder($sPath,$sExclFolders,$bShowHidden,$bShowSystem) Then
				_GUICtrlTreeView_AddChild($hTV,$hItem,$g_sFSF_NoFoldersMessage,$g_eFSF_IxBlank,$g_eFSF_IxBlank)
			EndIf
			_FSF_Expand_Item($hItem,$hTV,$sExclFolders,$bShowHidden,$bShowSystem)
			If @error Then
				SetError(1)
				Return
			EndIf
		Else
			_FSF_AddtoTree($hTV,$hItem,$a10Data,$sExclFolders,$bShowHidden,$bShowSystem)
		EndIf

		_GUICtrlTreeView_Expand($hTV,$hItem)
		_FSF_CollapseAllBlankItems($hTV)
		GUICtrlSendToDummy($g_cFSF_Expand_Dummy,0)	; de-select last item
		_GUICtrlTreeView_EnsureVisible($hTV,$hItemTop)
	EndIf
EndFunc

Func _FSF_InitializePerHybrid($hTV,$sRoot,$a10Desktop,$iendDT,$sExclFolders,$bShowHidden,$bShowSystem)
Local $s,$a10,$hItem,$sFileSysPath
	Local $a10sRoot = StringSplit($sRoot,'\',0+2)	; char,0-based

	Local $iEndAll = UBound($a10sRoot)-1	; index in $sRoot of last item
	Local $iEndDone = $iEndDT

	Local $bComputerEndsWithDrive=False,$bNetworkEndsWithDrive=False
	If $a10sRoot[$iEndDT]='Network' Then
		; Get names of PCs on the network
		If Not _FSF_IsPcOnline(   $a10sRoot[$iEndDone+1]) Then
			_FSF_ReportUserErroAndCondExit('$sRoot:"'&$a10sRoot[$iEndDone+1]&'" is not on the network')
			SetError(1)
			Return
		EndIf
		$iEndDone += 1

		If $iEndDone+1>$iEndAll Then
			_FSF_ReportUserErroAndCondExit('$sRoot: The share name on "'&$a10sRoot[$iEndDone]&'" is missing')
			SetError(1)
			Return
		EndIf

		GUICtrlSetData($g_cLblFSF_Info,'Searching for shares')
		Local $a10Shares = _FSF_GetShareNamesOnPC($a10sRoot[$iEndDone])
		GUICtrlSetData($g_cLblFSF_Info,'')
		If UBound($a10Shares)=0 Then
			_FSF_ReportUserErroAndCondExit('$sRoot: '&$a10sRoot[$iEndDone]&' has no shares')
			SetError(1)
			Return
		EndIf

		Local $bShareName=False,$sShareLtr=''
		For $i = 0 To UBound($a10Shares)-1
			; If root element is a share or share ends with (<element>)
			If $a10Shares[$i]=$a10sRoot[$iEndDone+1]  Then
				$bShareName = True
				ExitLoop
			EndIf
		Next

		If Not $bShareName Then
			$s = $a10sRoot[$iEndDone+1]
			If StringLen($s)=1 And StringIsAlpha($s) Then
				$sShareLtr = $s
			EndIf
		EndIf

		; If format is a letter
		If $sShareLtr<>'' Then
			For $i = 0 To UBound($a10Shares)-1
				If StringRight($a10Shares[$i],3)='('&$sShareLtr&')' Then
					; If at end of $sRoot
					If $iEndDone+1=$iEndAll Then
						$sRoot = StringTrimRight($sRoot,1)&$a10Shares[$i]
					Else
						$sRoot = _
							StringReplace($sRoot,'\'&$sShareLtr&'\','\'&$a10Shares[$i]&'\',1)	; first only
					EndIf
					$a10sRoot[$iEndDone+1] = $a10Shares[$i]
					$bShareName = True
					ExitLoop
				EndIf
			Next
		EndIf

		If Not $bShareName Then
			If $sShareLtr<>'' Then
				_FSF_ReportUserErroAndCondExit('$sRoot: "('&$sShareLtr&')" not found at end of a share name for "'& _
					$a10sRoot[$iEndDone]&'" so cannot replace it with a share name')
			Else
				_FSF_ReportUserErroAndCondExit('$sRoot: "'&$s&'" is not a share name on "'&$a10sRoot[$iEndDone]&'"')
			EndIf
		EndIf

		$sFileSysPath = '\\'&$a10sRoot[$iEndDone]&'\'&$a10sRoot[$iEndDone+1]	; UNC: \\computer\share
		$iEndDone += 1
		$bNetworkEndsWithDrive = $iEndDone=$iEndAll

		$g_sFSF_PreTree = 'Desktop\'

	ElseIf $a10sRoot[$iEndDT]='Computer' Then
		If $iEndDone+1>$iEndAll Then
			_FSF_ReportUserErroAndCondExit('$sRoot: The drive letter on "'&$a10sRoot[$iEndDone]&'" is missing')
			SetError(1)
			Return
		EndIf

		If Not StringRegExp($a10sRoot[$iEndDone+1],'^[A-Za-z]:$') Then
			_FSF_ReportUserErroAndCondExit('$sRoot: "'&$a10sRoot[$iEndDone+1]& _
				'" must be a drive letter (with a colon)')
			SetError(1)
			Return
		EndIf

		_FSF_ConnectComputerDrive($a10sRoot[$iEndDone+1])
		If @error Then
			SetError(1)
			Return
		EndIf

		$iEndDone += 1
		$bComputerEndsWithDrive = $iEndDone=$iEndAll

		$g_sFSF_PreTree = _FSF_GetPreTree($a10Desktop,$a10sRoot[0])
		$sFileSysPath = $a10sRoot[$iEndDone]
	Else	; Neither Network nor Computer
		$g_sFSF_PreTree = _FSF_GetPreTree($a10Desktop,$a10sRoot[0])
		$s = ''
		For $i = 0 To $iEndDT
			$s &= '\'&$a10sRoot[$i]
		Next
		$s = StringMid($s,2)
		$sFileSysPath = _FSF_GetPathForItem($s)
	EndIf

	Local $sProperCased
	For $i = $iEndDone+1 To $iEndAll
		$sProperCased = _FSF_CheckFolderInRoot($sFileSysPath,$a10sRoot[$i],$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return
		EndIf
		$sFileSysPath &= '\'&$sProperCased
	Next

	; Get icon
	Local $ixIcon
	; if $sRoot contains only  Desktop hierarchy items
	If $iEndAll=$iEndDT Then
		; Get icon from Desktop data array
		$a10 = _FSF_GetDataVectorForText($a10Desktop,$a10sRoot[$iEndDT])
		$ixIcon = $a10[1]
	; If Computer ... ends with a drive
	ElseIf $bComputerEndsWithDrive Then
		$a10 = _FSF_GetTextAndIconForComputerDrive($a10sRoot[$iEndAll])
		$ixIcon = $a10[1]
	ElseIf $bNetworkEndsWithDrive Then
		$ixIcon = $g_eFSF_IxRemoteDrive
	Else
		$ixIcon = $g_eFSF_IxFolder
	EndIf

	If $a10sRoot[0]='Computer' Then
		; Pick up text for local or mapped drive, replacing driveletter
		$a10 = _FSF_GetTextAndIconForComputerDrive($a10sRoot[1])
		$sRoot = StringReplace($sRoot,$a10sRoot[1],$a10[0],1)	; first only
	EndIf

	; Handle case where an element of $a10sRoot was proper-cased
	$sRoot = ''
	For $i = 0 To UBound($a10sRoot)-1
		$sRoot &= $a10sRoot[$i]&'\'
	Next
	$sRoot = StringTrimRight($sRoot,1)

	_GUICtrlTreeView_BeginUpdate($hTV)

	; Delete "Loading ..."
	_GUICtrlTreeView_DeleteAll($hTV)

	$hItem = _GUICtrlTreeView_Add($hTV,0,$sRoot,$ixIcon,$ixIcon)
	_GUICtrlTreeView_SetBold($hTV,$hItem)

	Local $iQitems = _GUICtrlTreeView_GetCount($hTV)
	_FSF_FillFolder($hTV,$hItem,$sFileSysPath,$sExclFolders, $bShowHidden,$bShowSystem)
	If @error Then
		SetError(1)
		Return
	EndIf
	If _GUICtrlTreeView_GetCount($hTV)=$iQitems Then
		_GUICtrlTreeView_AddChild($hTV,$hItem,$g_sFSF_NoFoldersMessage,$g_eFSF_IxBlank,$g_eFSF_IxBlank)
	EndIf
	_GUICtrlTreeView_Expand($hTV,$hItem)
	GUICtrlSendToDummy($g_cFSF_Expand_Dummy,0)	; de-select
	_FSF_CollapseAllBlankItems($hTV)
EndFunc

; Returns [<text>,<ixIcon>]
Func _FSF_GetTextAndIconForComputerDrive($sDrvLtr)
Local $a10Ret[2]
	$sDrvLtr = StringUpper($sDrvLtr)
	Local $a20map = _FSF_GetDriveMapping()
	Local $iN = -1
	For $i = 0 To UBound($a20map,1)-1
		If $a20map[$i][0]=$sDrvLtr Then
			$iN = $i
			ExitLoop
		EndIf
	Next
	If $iN>=0 Then
		$a10Ret[0] = $a20map[$iN][1]&' ('&$sDrvLtr&')'
		$a10Ret[1] = $g_eFSF_IxRemoteDrive
	EndIf

	If $a10Ret[0]='' Then
		Local $a11 = DriveGetDrive('CDROM')
		If @error=0 Then	; If computer has CD/DVD drive
			If $sDrvLtr=$a11[1] Then
				If DriveStatus($sDrvLtr)='READY' Then
					$a10Ret[0] = DriveGetLabel($sDrvLtr)&' ('&$sDrvLtr&')'
				Else
					$a10Ret[0] = 'BD-ROM Drive ('&$sDrvLtr&')'
				EndIf
				$a10Ret[1] = $g_eFSF_IxCD
			EndIf
		EndIf
	EndIf

	; Fixed, removable or RAM drive
	If $a10Ret[0]='' Then
		$a10Ret[0] = DriveGetLabel($sDrvLtr)&' ('&$sDrvLtr&')'
		If $sDrvLtr=StringLeft(@SystemDir,2) Then
			$a10Ret[1] = $g_eFSF_IxSysDrive
		Else
			$a10Ret[1] = $g_eFSF_IxLocalDrive
		EndIf
	EndIf
	Return $a10Ret
EndFunc

; To conform with FileSelectFolder, never shows hidden and system folders
Func _FSF_CreateDesktopHierarchyData($bNetwork,$sExclFolders)
	; Odd indenting is intentional. It shows relationships.
	; Tree data vector structure: [text, icon number, pre/post action, child1, child2, ... childN]
	;	action: 'b' = conditionally append blank-text child, '' = no-op
	If UBound($g_a10FSF_Desktop)<>0 Then
		Return $g_a10FSF_Desktop
	EndIf
			Local $a10MyDocs = ['My Documents',$g_eFSF_IxMyDocs,'b']
			Local $a10PubDocs = ['Public Documents',$g_eFSF_IxSpFolder,'b']
			Local $a10Docs = ['Documents',$g_eFSF_IxDocs,'',$a10MyDocs,$a10PubDocs]
			Local $a10MyMusic = ['My Music',$g_eFSF_IxMyMusic,'b']
			Local $a10PubMusic = ['Public Music',$g_eFSF_IxSpFolder,'b']
			Local $a10Music = ['Music',$g_eFSF_IxMusic,'',$a10MyMusic,$a10PubMusic]
			Local $a10MyPics = ['My Pictures',$g_eFSF_IxMyPics,'b']
				Local $a10PubPics =  ['Public Pictures',$g_eFSF_IxSpFolder,'b']
			Local $a10Pics = ['Pictures',$g_eFSF_IxPics,'',$a10MyPics,$a10PubPics]
			Local $a10MyVids = ['My Videos',$g_eFSF_IxMyVids,'b']
			Local $a10PubVids = ['Public Videos',$g_eFSF_IxSpFolder,'b']
			Local $a10Vids = ['Videos',$g_eFSF_IxVids,'',$a10MyVids,$a10PubVids]
		Local $a10Libs = ['Libraries',$g_eFSF_IxLibs,'',$a10Docs,$a10Music,$a10Pics,$a10Vids]

	Local $a10Computer = ['Computer',$g_eFSF_IxPC,'b']

	; Pick up folders in e.g. C:\Users\Chris
	Local $a10User[50] = [@UserName,$g_eFSF_IxUser,'']
	Local $sFnamext,$sAtts,$iQ=2,$a10[3]
	Local $hSearch = FileFindFirstFile(@HomeDrive&'\Users\'&@UserName&'\*.*')
	; If any folders or files
	If $hSearch<>-1 Then
		While 1
			$sFnamext = FileFindNextFile($hSearch,1)	; 1 = @extended contains attributes
			; If  no more folders/diles
			If @error Then
				ExitLoop
			EndIf
			$sAtts = @extended
			Switch True
			Case StringInStr($sAtts,'D')=0
				;do nothing
			Case StringInStr($sAtts,'H')
				; do nothing
			Case StringInStr($sAtts,'S')
				; do nothing
			Case $sExclFolders<>'' And StringInStr('|'&$sExclFolders&'|','|'&$sFnamext&'|')
				; do nothing
			Case Else
				If StringInStr('|Documents|Music|Pictures|Videos|','|'&$sFnamext&'|') Then
					Switch $sFnamext
						Case 'Documents'
							$a10 = $a10MyDocs
						Case 'Music'
							$a10 = $a10MyMusic
						Case 'Pictures'
							$a10 =$a10MyPics
						Case 'Videos'
							$a10 = $a10MyVids
					EndSwitch

					$a10User[$iQ] = $a10
					$iQ += 1
				Else
					$a10[0] = $sFnamext
					Switch $sFnamext
					Case 'Contacts'
						$a10[1] =$g_eFSF_IxConts
					Case 'Desktop'
						$a10[1] = $g_eFSF_IxUDT
					Case 'Downloads'
						$a10[1] = $g_eFSF_IxDLs
					Case 'Favorites'
						$a10[1] = $g_eFSF_IxFavs
					Case 'Links'
						$a10[1] = $g_eFSF_IxULinks
					Case Else
						$a10[1] = $g_eFSF_IxFolder
					EndSwitch
					$a10[2] = ''
					$a10User[$iQ] = $a10
					$iQ += 1
				EndIf

			EndSwitch
		WEnd
		FileClose($hSearch)
	EndIf
	ReDim $a10User[$iQ]

	; Insertion sort Ref: https://www.autoitscript.com/forum/topic/120139-1d-array-sorting-algorithms/
	Local $j,$a10t
	For $i = 3 To $iQ-1
		$j = $i
		$a10t = $a10User[$i]

		While $j+2>0 And ($a10User[$j-1])[0]>$a10t[0]
			$a10User[$j] = $a10User[$j-1]
			$j -= 1
		WEnd
		$a10User[$j] = $a10t
	Next

	If $bNetwork Then
		Local $a10Network = ['Network',$g_eFSF_IxNetwork,'b']
		Local $a10Desktop = ['Desktop',$g_eFSF_IxDT,'',$a10Libs,$a10Computer,$a10User,$a10Network]
	Else
		Local $a10Desktop = ['Desktop',$g_eFSF_IxDT,'',$a10Libs,$a10Computer,$a10User]
	EndIf
	$g_a10FSF_Desktop = $a10Desktop
	Return $g_a10FSF_Desktop
EndFunc

Func _FSF_InitializePerFileSpec($hTV,$sRoot,$sExclFolders, $bShowHidden,$bShowSystem)

	Local $a10sRoot = StringSplit($sRoot,'\',0+2)	; char,base 0
	_FSF_ConnectComputerDrive($a10sRoot[0])

	Local $sPath = $a10sRoot[0]

	; For folders
	Local $sProperCased
	For $i = 1 To UBound($a10sRoot)-1
		$sProperCased = _FSF_CheckFolderInRoot($sPath,$a10sRoot[$i],$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return
		EndIf
		$sPath &= '\'&$sProperCased
	Next

	Local $sFileSysPath = $sRoot

	Local $a10TI = _FSF_GetTextAndIconForComputerDrive($a10sRoot[0])
	$sRoot = StringReplace($sRoot,$a10sRoot[0],$a10TI[0],1)	; first only
	_GUICtrlTreeView_DeleteAll($hTV)	; removed "Loading ..."
	; If e.g. F:\ ...
	Local $ixIcon
	If UBound($a10sRoot)>1 Then
		$ixIcon = $g_eFSF_IxFolder
	Else
		$ixIcon = $a10TI[1]
	EndIf

	; Add drive item to treeview
	Local $hItem = _GUICtrlTreeView_Add($hTV,0,$sRoot,$ixIcon,$ixIcon)
	_GUICtrlTreeView_SetBold($hTV,$hItem)
	Local $hItemTop =  $hItem

	_GUICtrlTreeView_BeginUpdate($hTV)
	Local $iQitems = _GUICtrlTreeView_GetCount($hTV)
	_FSF_FillFolder($hTV,$hItem,$sFileSysPath,$sExclFolders, $bShowHidden,$bShowSystem)
	If @error Then
		SetError(1)
		Return
	EndIf
	_GUICtrlTreeView_Expand($hTV,$hItemTop,True)

	If _GUICtrlTreeView_GetCount($hTV)=$iQitems Then
		_GUICtrlTreeView_AddChild($hTV,$hItem,$g_sFSF_NoFoldersMessage,$g_eFSF_IxBlank,$g_eFSF_IxBlank)
	EndIf

	; Prevent creating a $g_cFSF_Expand_Dummy event which causes _FSF_Expand_Item to run.
	; _FSF_Expand_Item () then runs _FSF_FillFolder() to run again, erroneously
	GUICtrlSendToDummy($g_cFSF_Expand_Dummy,0)
	_FSF_CollapseAllBlankItems($hTV)
	_GUICtrlTreeView_EnsureVisible($hTV,$hItemTop)

	$g_sFSF_PreTree = 'Desktop\Computer\'
EndFunc

Func _FSF_InitializePerUNC($hTV,$sRoot,$sExclFolders, $bShowHidden,$bShowSystem)
; e.g. \\PETRA\PProgs (F)\AutoIt scripts
	Local $s = StringTrimLeft($sRoot,2)
	Local $a10sRoot = StringSplit($s,'\',0+2)	; char,base 0
	If UBound($a10sRoot)<2 Then
		_FSF_ReportUserErroAndCondExit('"'&$sRoot&'" is not a UNC path to a folder')
		SetError(1)
		Return
	EndIf

	Local $a10OnlinePCs = _FSF_GetOnLinePCs()
	If UBound($a10OnlinePCs)=0 Then
		_FSF_ReportUserErroAndCondExit('$sRoot: "'&$sRoot& _
			'" is a UNC path but either there is no network, or there are no other computers on the network')
		SetError(1)
		Return
	Else
		If Not _FSF_IsPcOnline(   $a10sRoot[0]) Then
			_FSF_ReportUserErroAndCondExit('$sRoot: "'&$a10sRoot[0]&'" is not on the network')
			SetError(1)
			Return
		EndIf
	EndIf

	GUICtrlSetData($g_cLblFSF_Info,'Searching for shares')
	Local $a10Shares = _FSF_GetShareNamesOnPC($a10sRoot[0])
	GUICtrlSetData($g_cLblFSF_Info,'')

	If UBound($a10Shares)=0 Then
		_FSF_ReportUserErroAndCondExit('$sRoot: No  shares on '&$a10sRoot[0]&' are accessible')
		SetError(1)
		Return
	EndIf

	Local $b = False
	For $i = 0 To UBound($a10Shares)-1
		; If $a10sRoot[1] is a share or ends with (<share>)
		If $a10sRoot[1]=$a10Shares[$i] Or StringRight($a10Shares[$i],3)='('&$a10sRoot[1]&')' Then
			$sRoot = StringReplace($sRoot,'\'&$a10sRoot[1],'\'&$a10Shares[$i],1) ; first only
			$a10sRoot[1] = $a10Shares[$i]
			$b = True
			ExitLoop
		EndIf
	Next

	If Not $b Then
		_FSF_ReportUserErroAndCondExit('$sRoot: Share "'&$a10sRoot[1]&'" on "'&$a10sRoot[0]&'" is not accessible')
		SetError(1)
		Return
	EndIf

	Local $sPath = '\\'&$a10sRoot[0]&'\'&$a10sRoot[1]	; computer, share
	; For folders
	Local $sProperCased
	For $i = 2 To UBound($a10sRoot)-1
		$sProperCased = _FSF_CheckFolderInRoot($sPath,$a10sRoot[$i],$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return
		EndIf
		$sPath &= '\'&$sProperCased
		$sRoot = $sPath
	Next

	_GUICtrlTreeView_BeginUpdate($hTV)

	; Handle case where an element of $a10sRoot was proper-cased
	$sRoot = '\\'
	For $i = 0 To UBound($a10sRoot)-1
		$sRoot &= $a10sRoot[$i]&'\'
	Next
	$sRoot = StringTrimRight($sRoot,1)

	_GUICtrlTreeView_DeleteAll($hTV)	; Delete 'Loading...'
	Local $ixIcon
	If UBound($a10sRoot)=2 Then
		$ixIcon = $g_eFSF_IxRemoteDrive
	Else
		$ixIcon = $g_eFSF_IxFolder
	EndIf

	Local $hItem = _GUICtrlTreeView_Add($hTV,0,$sRoot,$ixIcon,$ixIcon)
	_GUICtrlTreeView_SetBold($hTV,$hItem)
	Local $hItemTop = $hItem

	Local $iQitems = _GUICtrlTreeView_GetCount($hTV)
	_FSF_FillFolder($hTV,$hItem,$sRoot,$sExclFolders,$bShowHidden,$bShowSystem)
	If @error Then
		SetError(1)
		Return
	EndIf
	If _GUICtrlTreeView_GetCount($hTV)=$iQitems Then
		_GUICtrlTreeView_AddChild($hTV,$hItem,$g_sFSF_NoFoldersMessage,$g_eFSF_IxBlank,$g_eFSF_IxBlank)
	EndIf
	_GUICtrlTreeView_Expand($hTV,$hItem)
	; Prevent creating a $g_cFSF_Expand_Dummy event which causes _FSF_Expand_Item to run.
	; _FSF_Expand_Item () then runs _FSF_FillFolder() to run again, erroneously
	GUICtrlSendToDummy($g_cFSF_Expand_Dummy,0)
	_FSF_CollapseAllBlankItems($hTV)
	_GUICtrlTreeView_EnsureVisible($hTV,$hItemTop)

	$g_sFSF_PreTree = 'Desktop\Network\'
EndFunc

; Returns proper-cased $sFolder
Func _FSF_CheckFolderInRoot($sParent,$sFolder,$sExclFolders,$bShowHidden,$bShowSystem)
	Local $sRet = $sFolder

	Local $a10 = StringRegExp($sFolder,'['&$g_kFSF_sIllegalChars&']',1)
	If @error=0 Then
		_FSF_ReportUserErroAndCondExit('$sRoot: Illegal character '&$a10[0]&' in "'&$sFolder&'"')
		SetError(1)
		Return
	EndIf

	If StringRight($sFolder,1)='.' Then
		_FSF_ReportUserErroAndCondExit('$sRoot: "'&$sFolder&'" cannot end with a period')
		SetError(1)
		Return
	EndIf

	Local $sPath = $sParent&'\'&$sFolder
	Local $hndl = FileFindFirstFile($sPath)
	If $hndl=-1 Then
		_FSF_ReportUserErroAndCondExit('$sRoot: Folder "'&$sPath&'" does not exist')
		SetError(1)
		Return
	Else
		$sFolder = FileFindNextFile($hndl)	; proper-cased
		$sRet = $sFolder
		$sPath = $sParent&'\'&$sFolder
		FileClose($hndl)
	EndIf

	Local $sAtts = FileGetAttrib($sPath)
	If StringInStr($sAtts,'D')=0 Then
		_FSF_ReportUserErroAndCondExit('$sRoot: "'&$sPath&'" is a file, not a folder')
		SetError(1)
		Return
	EndIf

	If StringInStr('|'&$sExclFolders&'|','|'&$sFolder&'|')<>0 Then
		_FSF_ReportUserErroAndCondExit('"'&$sFolder&'" is in $sRoot but is excluded in $sExclFolders')
		SetError(1)
		Return
	EndIf

	If (Not $bShowHidden) And StringInStr($sAtts,'H') Then
		_FSF_ReportUserErroAndCondExit('$bShowHidden is False but "'&$sPath&'" in $sRoot is hidden')
		SetError(1)
		Return
	EndIf

	If (Not $bShowSystem) And StringInStr($sAtts,'S') Then
		_FSF_ReportUserErroAndCondExit('$bShowSystem is False but "'&$sPath&'" in $sRoot is a system folder')
		SetError(1)
		Return
	EndIf
	Return $sRet
EndFunc

Func _FSF_ConnectComputerDrive($sDrvLtr)
	; A mapped drive can be ready even if the share is not accesible
	Local $a20MappedDrives = _FSF_GetDriveMapping()
	Local $sShare=''
	For $i = 0 To UBound($a20MappedDrives,1)-1
		If $a20MappedDrives[$i][0]=$sDrvLtr Then
			$sShare = $a20MappedDrives[$i][1]
			ExitLoop
		EndIf
	Next
	Local $bMapped = $sShare<>''
	If $bMapped Then
		Local $a10 = StringSplit($sShare,'\',0+2)
		Local $sPC = $a10[2]
		Local $a10PCs = _FSF_GetOnLinePCs()
		Local $bOnNet = False
		For $i = 0 To UBound($a10PCs)-1
			If $a10PCs[$i]=$sPC Then
				$bOnNet = True
				ExitLoop
			EndIf
		Next
		If Not $bOnNet Then
			_FSF_ReportUserErroAndCondExit('$sRoot: Drive "'&$sDrvLtr&'" is on '&$sPC&', a computer that is not on the network')
			SetError(1)
			Return
		EndIf
		If Not _FSF_ConnectMappedDrive($sDrvLtr) Then
			_FSF_ReportUserErroAndCondExit('$sRoot: Cannot access mapped drive "'& _
				$sShare&' ('&$sDrvLtr&')"')
			SetError(1)
			Return
		EndIf
	Else
		Local $a11 = DriveGetDrive('CDROM')
		If @error=0 Then			; If computer has a CD/DVD drive
			Local $sCDdriveLetter = $a11[1]
			If $sDrvLtr=$sCDdriveLetter Then
				If FileExists($sDrvLtr&'\') Then
					Return
				Else
					If Not _FSF_DlgInsertCD($sDrvLtr) Then
						_FSF_ReportUserErroAndCondExit('$sRoot: "'&$sDrvLtr& _
							'" is a CD/DVD drive but user did not insert a disc')
						SetError(1)
						Return
					EndIf
					Return
				EndIf
			EndIf
		EndIf
		; Neither mapped nor CD drive
		If DriveStatus($sDrvLtr&'\')<>'READY' Then
			_FSF_ReportUserErroAndCondExit('$sRoot: Drive "'&$sDrvLtr&'" is unknown')
			SetError(1)
			Return
		EndIf
	EndIf
EndFunc

; Returns [[driveletter,driveText,ixIcon],[...
Func _FSF_GetComputerDrives_TextAndIcon()
	Local $sCDdriveLetter
	Local $a11All_DriveLetters = DriveGetDrive('FIXED')
	Local $a11 = DriveGetDrive('CDROM')
	If Not @error Then		; if a CD-DVD drive
		$a11All_DriveLetters = _FSF_MergeDriveLetters($a11All_DriveLetters,$a11)
		$sCDdriveLetter = $a11[1]
	EndIf

	$a11 = DriveGetDrive('REMOVABLE')
	If @error=0 Then
		$a11All_DriveLetters = _FSF_MergeDriveLetters($a11All_DriveLetters,$a11)
	EndIf

	$a11 = DriveGetDrive('RAMDISK')
	If @error=0 Then
		$a11All_DriveLetters = _FSF_MergeDriveLetters($a11All_DriveLetters,$a11)
	EndIf

	Local $iPosSt,$iPosEnd,$sPC
	Local $a20map = _FSF_GetDriveMapping()
	If UBound($a20map,1)<>0 Then
		GUICtrlSetData($g_cLblFSF_Info,'Connecting mapped drives')
	EndIf

	Local $a11[UBound($a20Map,1)+1],$iQ=0
	; e.g. \\PETRA\PDataH (G) => \\PETRA\PDataH (G0 (W:)
	; e.g. \\PETRA\PDataC (H)\temp => \\PETRA\PDataC (H)\temp (Q:)
	For $i = 0 To UBound($a20Map,1)-1
		; e.g. (\\petra\i) or (\\petra) => petra
		$iPosSt = StringInStr($a20Map[$i][1],'(\\')
		$sPC = StringTrimLeft($a20map[$i][1],$iPosSt+2)
		$iPosEnd = StringInStr($sPC,'\')
		If $iPosEnd=0 Then
			$iPosEnd = StringInStr($sPC,')')
		EndIf
		$sPC = StringLeft($sPC,$iPosEnd-1)

		If _FSF_IsPcOnline(   $sPC) Then
		; Also connect CD drive if disc in it
			If DriveStatus($a20Map[$i][0]&'\')<>'READY' Then
				_FSF_ConnectMappedDrive($a20Map[$i][0])
			EndIf
			If DriveStatus($a20Map[$i][0]&'\')='READY' Then
				$iQ += 1
				$a11[$iQ] = $a20Map[$i][0]
			EndIf
		EndIf
	Next
	ReDim $a11[$iQ+1]
	$a11[0] = $iQ

	If $iQ>0 Then
		$a11All_DriveLetters = _FSF_MergeDriveLetters($a11All_DriveLetters,$a11)
	EndIf

	; Get texts and icons for drives
	Local $sMappedDrives=''
	For $i = 0 To UBound($a20Map,1)-1
		$sMappedDrives &= $a20Map[$i][0]
	Next

	Local $a20Ret[($a11All_DriveLetters[0])][3]
	Local $sDriveLetter,$sText,$iIxIcon
	For $i = 1 To $a11All_DriveLetters[0]
		$sDriveLetter = StringUpper($a11All_DriveLetters[$i])
		$sText=''
		If $sDriveLetter=$sCDdriveLetter Then
			If DriveStatus($sDriveLetter)='READY' Then
				$sText = DriveGetLabel($sDriveLetter)&' ('&$sDriveLetter&')'
			Else
				$sText = 'BD-ROM Drive ('&$sDriveLetter&')'
			EndIf
			$iIxIcon = $g_eFSF_IxCD
		ElseIf StringInStr($sMappedDrives,$sDriveLetter) Then
			For $iJ = 0 To UBound($a20Map,1)-1
				If $a20Map[$iJ][0]=$sDriveLetter Then
					$sText =  $a20Map[$iJ][1]&' ('&$sDriveLetter&')'
					ExitLoop
				EndIf
			Next
			$iIxIcon = $g_eFSF_IxMapped
		Else
			$sText = DriveGetLabel($sDriveLetter)&' ('&$sDriveLetter&')'
			If $sDriveLetter=StringLeft(@SystemDir,2) Then
				$iIxIcon = $g_eFSF_IxSysDrive
			Else
				$iIxIcon = $g_eFSF_IxLocalDrive
			EndIf
		EndIf
		$a20Ret[$i-1][0] = $sDriveLetter
		$a20Ret[$i-1][1] = $sText
		$a20Ret[$i-1][2] = $iIxIcon
	Next
	GUICtrlSetData($g_cLblFSF_Info,'')
	Return $a20Ret
EndFunc

; To be recognized, folders must meet the hidden/system criteria
Func _FSF_DoesFolderContainFolder($sPath,$sExclFolders,$bShowHidden,$bShowSystem)
Local $bRet
	If StringRight($sPath,1)<>'\' Then
		$sPath &= '\'
	EndIf

	Local $sAtts,$sFnamext
	Local $iSearch = FileFindFirstFile($sPath&'*.*')
	If $iSearch=-1 Then	; no files found
		$bRet = False
	Else
		While 1
			$sFnamext = FileFindNextFile($iSearch,1)	; 1 = @extended contains result of
			If @error Then
				ExitLoop
			EndIf
			$sAtts = @extended
			Switch True
			Case StringInStr($sAtts,'D')=0
				;do nothing
			Case (Not $bShowHidden) And StringInStr($sAtts,'H')
				; do nothing
			Case (Not $bShowSystem) And StringInStr($sAtts,'S')
				; do nothing
			Case $sExclFolders<>'' And StringInStr('|'&$sExclFolders&'|','|'&$sFnamext&'|')
				; do nothing
			Case Else
				$bRet = True
				ExitLoop
			EndSwitch
		WEnd
		FileClose($iSearch)
	EndIf
	Return $bRet
EndFunc

; PreTree is the chain of items from Desktop down to, but not including, $text
; $a10Data is always $a10Desktop
Func _FSF_GetPreTree($a10data,$text)
	If $text='Desktop' Then
		Return ''
	ElseIf $text='Network' Then
		Return 'Desktop\'
	EndIf

	Local $s =  _FSF_GetPreTree_Sub($a10data,$text)
	Local $iPos = StringInStr($s,'\',0,-1)	; last
	If $iPos<>0 Then
		$s = Stringleft($s,$iPos)	; including trailing backslash
	Else
		$s=''
	EndIf
	Return $s
EndFunc

Func _FSF_GetPreTree_Sub($a10data,$text)
	If $a10data[0]=$text Then
		Return $a10data[0]
	EndIf
	Local $s
	For $i = 3 To UBound($a10data)-1
		$s =  _FSF_GetPreTree_Sub($a10data[$i],$text)
		If $s<>'' Then
			Return $a10data[0]&'\'&$s
		EndIf
	Next
	Return ''
EndFunc

; By recursing through the Desktop data array, finds data vector satisfying tree
;
; Parameters:
;
; $a10Data	starting data vector: [text, icon index, action] action='b' inserts blank line after treeview item
; $sTree	tree path to search for in data vectors, e.g.  Desktop\Libraries\Documents\My Documents
;
; Success: returns data vector
; Failure: returns empty vector

Func _FSF_GetDataVectorForTree($a10data,$sTree)
	Local $a10sTree = StringSplit($sTree,'\',0+2)
	Local $a10DT = $a10data
	Local $a10
	For $i = 0 To UBound($a10sTree)-1
		$a10 = _FSF_GetDataVectorForText($a10DT,$a10sTree[$i])
		$a10DT = $a10
	Next
	Return $a10DT
EndFunc

Func _FSF_GetDataVectorForText($a10data,$text)
Local $a1Null[0],$aRet
	If UBound($a10data)<>0 And $a10data[0]=$text Then
		Return $a10data
	EndIf
	For $i = 3 To UBound($a10data)-1
		$aRet = _FSF_GetDataVectorForText($a10data[$i],$text)
		If UBound($aRet)<>0 Then
			Return $aRet
		EndIf
	Next
	Return $a1Null
EndFunc

Func _FSF_AddtoTree($hTV,$hParent,$a10Cur,$sExclFolders,$bShowHidden,$bShowSystem)
Local $a10Child,$hItem,$sPath
	For $i = 3 To UBound($a10Cur)-1
		$a10Child = $a10Cur[$i]
		$hItem = _GUICtrlTreeView_AddChild($hTV,$hParent,$a10Child[0],$a10Child[1],$a10Child[1])
		If $a10Child[2]='b' Then
			$sPath = _FSF_GetPathForItem($hItem)
			If _FSF_DoesFolderContainFolder($sPath,$sExclFolders,$bShowHidden,$bShowSystem) Then
				; Dummy item so [+] shows when no subfolders/items yet
				_GUICtrlTreeView_AddChild($hTV,$hItem,'',$g_eFSF_IxBlank,$g_eFSF_IxBlank)
			EndIf
		EndIf
		_FSF_AddtoTree($hTV,$hItem,$a10Child,$sExclFolders,$bShowHidden,$bShowSystem)
	Next
EndFunc

Func _FSF_Expand_Item($hItem,$hTV,$sExclFolders,$bShowHidden,$bShowSystem)
Local $a10,$hFirstChild,$sFirstChild,$h,$bUpdatingTV=False,$s

	Local $sTree = $g_sFSF_PreTree&StringReplace(_GUICtrlTreeView_GetTree($hTV, $hItem),'|','\')
	Local $a10Tree = StringSplit($sTree,'\',0+2)	; char,0-based
	Local $sCDdriveLetter=''
	$a10 = DriveGetDrive('CDROM')
	If @error=0 Then
		$sCDdriveLetter = $a10[1]
	EndIf

	If $sTree='Desktop\Computer' Then
		; Add drives
		$hFirstChild = _GUICtrlTreeView_GetFirstChild($hTV, $hItem)
		$sFirstChild = _GUICtrlTreeView_GetText($hTV, $hFirstChild)
		If $sFirstChild='' Then
			If Not $g_bFSF_SelectingInitialItem Then
				_GUICtrlTreeView_BeginUpdate($hTV)
			EndIf
			Local $a20 =_FSF_GetComputerDrives_TextAndIcon()
			For $i = 0 To UBound($a20,1)-1
				$h = _GUICtrlTreeView_AddChild($hTV,$hItem,$a20[$i][1],$a20[$i][2],$a20[$i][2])
				; dummy, so [*] shows and know that PC not expanded
				_GUICtrlTreeView_AddChild($hTV,$h,'',$g_eFSF_IxBlank,$g_eFSF_IxBlank)
			Next
			_FSF_EnsureTopOfExpansionIsVisible($hTV,$hItem)
			_GUICtrlTreeView_Delete($hTV, $hFirstChild)			; Delete the dummy
			If Not $g_bFSF_SelectingInitialItem Then
				_GUICtrlTreeView_EndUpdate($hTV)
			EndIf
		EndIf
		Return
	ElseIf $sTree='Desktop\Computer\BD-ROM Drive ('&$sCDdriveLetter&')' Then
		; Request disc insertion and reflect it in drive name
		$hFirstChild = _GUICtrlTreeView_GetFirstChild($hTV, $hItem)
		$sFirstChild = _GUICtrlTreeView_GetText($hTV, $hFirstChild)
		If Not FileExists($sCDdriveLetter&'\') Then
			_GUICtrlTreeView_Expand($hTV,$hItem,False)	; collapse
			If Not $g_bFSF_SelectingInitialItem Then
				_GUICtrlTreeView_BeginUpdate($hTV)
			EndIf
			$bUpdatingTV = True
			If Not _FSF_DlgInsertCD($sCDdriveLetter) Then
				Return
			EndIf
			$s = DriveGetLabel($sCDdriveLetter&'\')
			_GUICtrlTreeView_SetText($hTV,$hItem,$s&' ('&StringUpper($sCDdriveLetter)&')')
		EndIf
	ElseIf $sTree='Desktop\Network' Then
		; Add computers
		$hFirstChild = _GUICtrlTreeView_GetFirstChild($hTV, $hItem)
		$sFirstChild = _GUICtrlTreeView_GetText($hTV, $hFirstChild)
		If $sFirstChild='' Then
			Local $a10OnlinePCs = _FSF_GetOnLinePCs()
			For $i = 0 To UBound($a10OnlinePCs)-1
				$h = _GUICtrlTreeView_AddChild($hTV,$hItem,$a10OnlinePCs[$i],$g_eFSF_IxPC,$g_eFSF_IxPC)
				; dummy, so [*] shows and know that PC not expanded
				_GUICtrlTreeView_AddChild($hTV,$h,'',$g_eFSF_IxBlank,$g_eFSF_IxBlank)
			Next
			_FSF_EnsureTopOfExpansionIsVisible($hTV,$hItem)
			_GUICtrlTreeView_Delete($hTV, $hFirstChild)			; Delete the dummy
			If Not $g_bFSF_SelectingInitialItem Then
				_GUICtrlTreeView_EndUpdate($hTV)
			EndIf
		EndIf
		Return
	ElseIf UBound($a10Tree)=3 And $a10Tree[1]='Network'	Then ; e.g. Desktop|Network|Petra
		; Add remote drives
		$hFirstChild = _GUICtrlTreeView_GetFirstChild($hTV, $hItem)
		$sFirstChild = _GUICtrlTreeView_GetText($hTV, $hFirstChild)
		If $sFirstChild='' Then
			Local $sPC = $a10Tree[2]
			If Not $g_bFSF_SelectingInitialItem Then
				_GUICtrlTreeView_BeginUpdate($hTV)
			EndIf
			GUICtrlSetData($g_cLblFSF_Info,'Searching for shares')
			Local $a10Shares = _FSF_GetShareNamesOnPC($sPC)
			GUICtrlSetData($g_cLblFSF_Info,'')
			If UBound($a10Shares)<>0 Then
				For $iDr = 0 To UBound($a10Shares)-1
					$h = _GUICtrlTreeView_AddChild($hTV,$hItem,$a10Shares[$iDr],$g_eFSF_IxRemoteDrive,$g_eFSF_IxRemoteDrive)
					; dummy, so [*] shows and know that share not expanded
					_GUICtrlTreeView_AddChild($hTV,$h,'',$g_eFSF_IxBlank,$g_eFSF_IxBlank)
				Next
				; Delete the dummy
				_GUICtrlTreeView_Delete($hTV, $hFirstChild)
				_FSF_EnsureTopOfExpansionIsVisible($hTV,$hItem)
				If Not $g_bFSF_SelectingInitialItem Then
					_GUICtrlTreeView_EndUpdate($hTV)
				EndIf
			Else
				GUICtrlSetData($g_cLblFSF_Info,$sPC&' is not on the network')
				Sleep(3000)
				GUICtrlSetData($g_cLblFSF_Info,'')
				; Delete the dummy
				_GUICtrlTreeView_Delete($hTV, $hFirstChild)
			EndIf
			Return
		EndIf
	EndIf

	; Check if dummy child exists or has already been filled
	$hFirstChild = _GUICtrlTreeView_GetFirstChild($hTV, $hItem)
	$sFirstChild = _GUICtrlTreeView_GetText($hTV, $hFirstChild)
	; If dummy child exists
	If $sFirstChild = "" Then
		; Fill with content
		Local $sPath = _FSF_GetPathForItem($hItem)
		If StringRegExp($sPath,'^[A-Z]\:$') Then	; e.g. W:
			If DriveStatus($sPath)<>'READY' Then	; disconnected
				_FSF_ConnectMappedDrive($sPath)
				If DriveStatus($sPath)<>'READY' Then
					_GUICtrlTreeView_Delete($g_hFSF_TV,$hFirstChild)
					_FSF_VarSizeDialog('Drive inaccessible','Drive '&$sPath&' is not accessible',0)
					Return
				EndIf
			EndIf
		EndIf
		_GUICtrlTreeView_Delete($hTV, $hFirstChild)
		If $bUpdatingTV Then
			If Not $g_bFSF_SelectingInitialItem Then
				_GUICtrlTreeView_EndUpdate($hTV)
			EndIf
		EndIf
		_FSF_FillFolder($hTV,$hItem,$sPath,$sExclFolders,$bShowHidden,$bShowSystem)
		If @error Then
			SetError(1)
			Return
		EndIf
		_FSF_EnsureTopOfExpansionIsVisible($hTV,$hItem)
	EndIf
EndFunc

; Adds folders below a drive or folder treeview item
Func _FSF_FillFolder($hTV, $hParent, $sPath, $sExclFolders, $bShowHidden,$bShowSystem)
Local $a10SubPaths[100],$a10PaItems[100],$iHiSub=-1,$iN
	If StringRight($sPath,1)<>'\' Then
		$sPath &= '\'
	EndIf

	If Not FileExists($sPath) Then
		_FSF_ReportUserErroAndCondExit('$sRoot: Folder "'&$sPath&'" does not exist')
		SetError(1)
		Return
	EndIf

	Local $sFnamext,$sAtts,$sAttsSub,$sItem,$hItem,$iSearchSub
	Local $bh,$bs,$bi
	Local $iSearch = FileFindFirstFile($sPath&'*.*')
	If $iSearch<>-1 Then	; files found
		While 1
			$sFnamext = FileFindNextFile($iSearch,1)	; 1 = @extended contains result of
			If @error Then
				ExitLoop
			EndIf
			$sAtts = @extended
			If StringInStr($sAtts,'D') Then
				If StringInStr($sAtts,'H') Then
					$bh = $bShowHidden
				Else
					$bh = True
				EndIf
				If StringInStr($sAtts,'S') Then
					$bs = $bShowSystem
				Else
					$bs = True
				EndIf
				$bi = StringInStr('|'&$sExclFolders&'|','|'&$sFnamext&'|')=0
				If $bh And $bs And $bi Then
					$sItem = $sFnamext
					If $hParent=$hTV Then
						$hItem = _GUICtrlTreeView_Add($hTV,$hParent,$sItem,$g_eFSF_IxFolder,$g_eFSF_IxFolder)
					Else
						$hItem = _GUICtrlTreeView_AddChild($hTV,$hParent,$sItem,$g_eFSF_IxFolder,$g_eFSF_IxFolder)
					EndIf
					$iHiSub += 1
					$iN = UBound($a10SubPaths)
					If $iHiSub>$iN Then
						$iN += 100
						ReDim $a10SubPaths[$iN]
						ReDim $a10PaItems[$iN]
					EndIf
					$a10SubPaths[$iHiSub] = $sPath&$sFnamext&'\*.*'
					$a10PaItems[$iHiSub] = $hitem
				EndIf
			EndIf
		WEnd
		FileClose($iSearch)

		_SendMessage($g_hFSF_TV, $TVM_SORTCHILDREN, 0, $hParent, 0, "wparam", "handle")

		; Show expansion + if each folder contains folder(s) that meet criteria
		For $i = 0 To $iHiSub
			$iSearchSub = FileFindFirstFile($a10SubPaths[$i])
			If $iSearchSub=-1 Then
				; empty subdir
			Else
				While 1
					$sFnamext = FileFindNextFile($iSearchSub,1)
					If @error Then
						ExitLoop
					EndIf
					$sAttsSub = @extended
					If StringInStr($sAttsSub,'D') Then
						If StringInStr($sAttsSub,'H') Then
							$bh = $bShowHidden
						Else
							$bh = True
						EndIf
						If StringInStr($sAttsSub,'S') Then
							$bs = $bShowSystem
						Else
							$bs = True
						EndIf
						$bi = StringInStr('|'&$sExclFolders&'|','|'&$sFnamext&'|')=0

						If $bh And $bs And $bi Then
							; Create dummy child to force [+] display and as marker for filling subdir
							_GUICtrlTreeView_AddChild($hTV,$a10PaItems[$i],'',$g_eFSF_IxBlank,$g_eFSF_IxBlank)
							ExitLoop
						EndIf
					EndIf
				WEnd
				FileClose($iSearchSub)
			EndIf
		Next
	EndIf
EndFunc

; For a tree, returns file-system equivalent. If Desktop item has none, returns ''
;
; This function searches for $g_sFSF_PreTree&$sTree

; $g_sFSF_PreTree is the part of the Desktop hierarchy that is higher than what the treeview shows.

; $vItem can be either $sTree (a string) or, if $vitem is the handle of treeview item, this function
; calls _GUICtrlTreeView_GetTree() and converts the result to $sTree

Func _FSF_GetPathForItem($vItem)
Local $sRet='-- unhandled --',$sTreePath,$sTree,$s
	If IsString($vItem) Then
		$sTree = $vItem
	Else
		$sTree = StringReplace(_GUICtrlTreeView_GetTree($g_hFSF_TV,$vItem),'|','\')	; replace all
	EndIf

	$sTreePath = $g_sFSF_PreTree&$sTree

	$sTreePath = StringReplace($sTreePath,'Network\\\','Network\',1)

	Local $a10 = StringSplit($sTreePath,'\',0+2)	; char,0-based
	If $sTreePath='Desktop' Then
		$sRet = ''
	ElseIf $a10[0]='Desktop' Then
		If $a10[1]='Libraries' Then
			If $sTreePath='Desktop\Libraries' Then
				$sRet = ''
			ElseIf $a10[2]='Documents' Then
				If $sTreePath='Desktop\Libraries\Documents' Then
					$sRet = ''	; Does not support libraries
				ElseIf $a10[3]='My Documents' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Documents\My Documents', _
						@HomeDrive&@HomePath&'\Documents') ; e.g. C:\Users\Chris
				ElseIf $a10[3]= 'Public Documents' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Documents\Public Documents', _
						@HomeDrive&'\Users\Public\Documents')
				EndIf
			ElseIf $a10[2]='Music' Then
				If $sTreePath='Desktop\Libraries\Music' Then
					$sRet = ''
				ElseIf $a10[3]='My Music' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Music\My Music', _
						@HomeDrive&@HomePath&'\Music') ; e.g. C:\Users\Chris
				ElseIf $a10[3]='Public Music' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Music\Public Music', _
						@HomeDrive&'\Users\Public\Music')
				EndIf
			ElseIf $a10[2]='Pictures' Then
				If $sTreePath='Desktop\Libraries\Pictures' Then
					$sRet = ''
				ElseIf $a10[3]='My Pictures' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Pictures\My Pictures', _
						@HomeDrive&@HomePath&'\Pictures') ; e.g. C:\Users\Chris
				ElseIf $a10[3]='Public Pictures' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Pictures\Public Pictures', _
						@HomeDrive&'\Users\Public\Pictures')
				EndIf
			ElseIf $a10[2]='Videos' Then
				If $sTreePath='Desktop\Libraries\Videos' Then
					$sRet = ''
				ElseIf $a10[3]='My Videos' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Videos\My Videos', _
						@HomeDrive&@HomePath&'\Videos')
				ElseIf $a10[3]='Public Videos' Then
					$sRet = StringReplace($sTreePath,'Desktop\Libraries\Videos\Public Videos', _
						@HomeDrive&'\Users\Public\Videos')
				EndIf
			Else
				; 'can't happen
			EndIf
		ElseIf $a10[1]='Homegroup' Then
			$sRet = ''
		ElseIf $a10[1]=@UserName Then
			If $sTreePath='Desktop\'&@UserName Then
				$sRet = @HomeDrive&'\Users\'&@UserName
			Else
				$sRet = StringReplace($sTreePath,'Desktop',@HomeDrive&'\Users',1)	; 1st only
				Switch $a10[2]
					Case 'My Documents','My Music','My Pictures','My Videos'
						$sRet = StringReplace($sRet,$a10[2],StringMid($a10[2],4),1)	; remove 'My '
				EndSwitch
			EndIf
		ElseIf $a10[1]='Computer' Then
			If $sTreePath='Desktop\Computer' Then
				$sRet = ''
			Else
				Local $iPos = StringInStr($sTreePath,':')
				If StringMid($sTreePath,$iPos+1,1)=')' Then	; if labelled drive e.g. MDataH (G:)
					$sRet = StringMid($sTreePath,$iPos-1,1)&':'&StringMid($sTreePath,$iPos+2)	; drop drive label
				ElseIf $iPos=2 Then
					$sRet = StringMid($sTreePath,$iPos-1,1)&':\'&StringMid($sTreePath,$iPos+2)
				ElseIf $ipos<>0 Then
					$s = _FSF_GetDriveLetterForLabel(StringLeft($sTreePath,$iPos-1))
					$sRet = $s&StringMid($sTreePath,$iPos)
				EndIf
			EndIf
		ElseIf $a10[1]='Network' Then
			If $sTreePath='Desktop\Network' Then
				$sRet = ''
			ElseIf UBound($a10)=3 Then	; e.g. 'Desktop','Network','Molly'
				$sRet = ''
			ElseIf $a10[2]=@ComputerName Then	; e.g. Molly\MDataC (H)
				Local $a20Lsp = _FSF_GetLocalShareNamesAndPaths()
				Local $sPath
				For $i = 0 To UBound($a20Lsp,1)-1
					If $a20Lsp[$i][0]=$a10[3] Then
						$sPath = $a20Lsp[$i][1]
						ExitLoop
					EndIf
				Next
				If StringRight($sPath,1)<>'\' Then
					$sPath &= '\'
				EndIf
				; Convert to local path
				$sRet = $sPath
				For $i = 4 To UBound($a10)-1
					$sRet &= $a10[$i]&'\'
				Next
				While StringRight($sRet,1)='\'
					$sRet = StringTrimRight($sRet,1)
				WEnd
			Else
				$sRet = '\\'&StringMid($sTreePath,17)	; drop "Desktop\Network\" so begins "\\Petra"
			EndIf
		ElseIf $a10[1]='Control Panel' Then
			$sRet = ''
;			not implemented
		EndIf
	EndIf

	If StringInStr($sTree,$g_sFSF_NoFoldersMessage) Then
		$sRet = ''
	EndIf

	If $sRet='-- unhandled --' Then
		_FSF_VarSizeDialog('_cFileSelectFolder Coding error','Unhandled item '&@CRLF&@CRLF& _
		'$g_sFSF_PreTree = "'&$g_sFSF_PreTree&'"'&@CRLF&'$sTree = "'&$sTree&'"',0)
		Exit
	EndIf
	Return $sRet
EndFunc

Func _FSF_GetDriveLetterForLabel($sLbl)
	Local $aRet=''
	Local $a11 = DriveGetDrive('ALL')
	For $i = 1 To UBound($a11)-1
		If DriveGetLabel($a11[$i]&'\')=$sLbl Then
			$aRet = $a11[$i]
			ExitLoop
		EndIf
	Next
	Return $aRet
EndFunc

Func _FSF_ConnectMappedDrive($sDrive)
Local $bRet=False
; Ref: https://www.autoitscript.com/forum/topic/71790-enum-network-connections/
	Local $host = "."
	; Using these constants didn't seem to help
;~ 	Local Const $kWbemFlagReturnImmediately = 0x10
;~ 	Local Const $kWbemFlagForwardOnly = 0x20
	Local $objWMIService = ObjGet("winmgmts:{impersonationLevel=impersonate}!\\" & $host & "\root\cimv2")
	Local $oWMIdriveInfo = $objWMIService.ExecQuery("SELECT * from Win32_NetworkConnection")
	For $objItem In $oWmiDriveInfo
		If $objItem.LocalName=$sDrive Then
			If $objItem.ConnectionState='Connected' Then
				$bRet = True
			Else
				; not persistent,Show authentication dialog if required. This is a kludge for connect!
				DriveMapAdd($objItem.LocalName, $objItem.RemoteName, 8)
				$bRet = @error=0
			EndIf
			ExitLoop
		EndIf
	Next
	Return $bRet
EndFunc

Func _FSF_ReportUserErroAndCondExit($sMsg)
	If IsDeclared('g_bFSF_ReturnUserErrorMessage')<>$DECLARED_UNKNOWN Then
		$g_sFSF_UserErrorMessage = '_cFileSelectFolder User error'&@CRLF&@CRLF&$sMsg
	Else
		_FSF_VarSizeDialog('_cFileSelectFolder: User error',$sMsg&@CRLF&@CRLF&'Script will exit',0)
		Exit
	EndIf
EndFunc

; Returns: base 1 vector, e.g. [2,'C:','E:']
; Letters in inputs can be lower case.
; Inputs can be no drives, e.g. [0] or ''
Func _FSF_MergeDriveLetters($a11A,$a11b)
	If $a11A='' Then
		Local $a11[1]
		$a11[0] = 0
		$a11A = $a11
	EndIf

	If $a11b='' Then
		Local $a11[1]
		$a11[0] = 0
		$a11b = $a11
	EndIf

	Local $flag=0,$iQ = $a11A[0]
	For $i = 1 To $a11A[0]
		$flag = BitOR($flag,BitShift(1,-(Asc(StringUpper(StringLeft($a11A[$i],1)))-Asc('A'))))
	Next

	Local $iN
	For $i = 1 To $a11b[0]
		$iN = BitShift(1,-(Asc(StringUpper(StringLeft($a11b[$i],1)))-Asc('A')))
		If BitAND($flag,$iN)=0 Then
			$flag = BitOR($flag,$iN)
			$iQ += 1
		EndIf
	Next
	Local $a11Ret[$iQ+1]
	$a11Ret[0] = $iQ
	Local $iJ = 0

	For $i =  0 To 25
		If BitAND($flag,BitShift(1,-$i)) Then
			$iJ += 1
			$a11Ret[$iJ] = Chr($i+Asc('A'))&':'
			If $iJ=$iQ Then
				ExitLoop
			EndIf
		EndIf
	Next
	Return $a11Ret
EndFunc

; Returns names of PCs in 1-dimension array[<PC1>,,PC2>, ...]
; Includes @ComputerName, e.g. Molly

Func _FSF_GetOnLinePCs()
	Local $a10Ret = _FSF_EnumPCs()

	; It is likely that EnumPCs finds PCs the router knows about, so we ping to check PCs are on line
	Local $iQ=0
	For $i = 0 To UBound($a10Ret)-1
	;	cmd line ping is much slower, even where remote computer is accessible, and it too does not obey timeout argument
		Ping($a10Ret[$i],500)	; wait 500 mSecs for it to ping back
		If @error=0 Then
			$a10Ret[$iQ] = $a10Ret[$i]
			$iQ += 1
		EndIf
	Next
	ReDim $a10Ret[$iQ]

	Return $a10Ret
EndFunc

; Proper-cases $sPC
Func _FSF_IsPcOnline(ByRef $sPC)
	Local $a10 = _FSF_EnumPCs()

	; It is likely that EnumPCs finds PCs the router knows about, so we ping to check PC is on line
	Local $bRet = False
	For $i = 0 To UBound($a10)-1
		If $a10[$i]=$sPC Then
			$sPC = $a10[$i]	; proper-case it
			Ping($sPC,500)	; wait 500 mSecs for it to ping back
			If @error=0 Then
				$bRet = True
				ExitLoop
			EndIf
		EndIf
	Next

	Return $bRet
EndFunc

Func _FSF_IsAnyPcOnline()
	Local $a10 = _FSF_EnumPCs()				; $a10[0]=@ComputerName
	; It is likely that EnumPCs finds PCs the router knows about, so we ping to check PC is on line
	Local $bRet=False
	For $i = 0 To UBound($a10)-1
		If $a10[$i]<>@ComputerName Then
			Ping($a10[$i],500)	; wait 500 mSecs for it to ping back
			If @error=0 Then
				$bRet =True
				ExitLoop
			EndIf
		EndIf
	Next
	Return $bRet
EndFunc

Func _FSF_EnumPCs()
Local $a10Ret[0]
	; It is likely that EnumPCs finds PCs the router knows about
	Local $tResource = DllStructCreate($tagNETRESOURCE)
	Local $hEnum
	Local $iBufSize = 16384
	Local $pBuffer = _MemGlobalAlloc($iBufSize)
	Local $iCount = -1
	_FSF_cWinNet_OpenEnum($RESOURCE_CONTEXT,$RESOURCETYPE_DISK,$RESOURCEUSAGE_CONTAINER,$tResource,$hEnum)

	_WinNet_EnumResource($hEnum,  $iCount,$pBuffer,  $iBufSize)
	Local $pEnum = $pBuffer

	; The first resource is Entire Network, a container
	ReDim $a10Ret[$iCount-1]
	Local $tstruct
	$pEnum = $pBuffer+32
	For $i = 2 To $iCount
		$tstruct = DllStructCreate($tagNETRESOURCE,$pEnum)
		$a10Ret[$i-2] =StringTrimLeft(_FSF_PointerToStringW(DllStructGetData($tstruct,'RemoteName')),2)
		$pEnum += 32
	Next
	_WinNet_CloseEnum($hEnum)
	_MemGlobalFree($pBuffer)
	Return $a10Ret
EndFunc

Func _FSF_CollapseAllBlankItems($hTV)
	Local $sItem,$hParent

	Local $hItem = _GUICtrlTreeView_GetFirstItem($hTV)
	While $hItem <> 0x00000000
		$sItem = _GUICtrlTreeView_GetText($hTV,$hItem)
		If $sItem='' Then
			$hParent = _GUICtrlTreeView_GetParentHandle($hTV,$hItem)
			If $hParent<>0x00000000 Then
				_GUICtrlTreeView_Expand($hTV,$hParent,False)
			EndIf
		EndIf
		$hItem = _GUICtrlTreeView_GetNext($hTV, $hItem)
	WEnd
EndFunc

; Returns True or False
; Waits for CD/DVD to spin up unless user clicks on Cancel button
Func _FSF_DlgInsertCD($sDrvLtr)
	Local $iLeft,$iTop
	_FSF_GetLeftTopForCentringInParent(341,139,  $iLeft,  $iTop)

#Region ### START Koda GUI section ### Form=F:\AutoIt scripts\List files in a path\frmInsertDisc.kxf  MODIFIED
Local $frmInsertDisc = GUICreate("Insert a disc", 341, 139, $iLeft, $iTop,-1,-1,$g_hFSF_TV)
GUISetFont(8.5,400,0,'Tahoma',Default,5)
;~ Local $frmInsertDisc = GUICreate("Insert a disc", 341, 139, 192, 114)
GUICtrlCreateLabel("Insert a disc", 80, 32, 74, 17)
GUICtrlSetFont(-1, 8, 800, 0, "MS Sans Serif")
GUICtrlCreateLabel("Please insert a disc into CD-DVD drive", 80, 56, 185, 17)
Local $cbtnCancel = GUICtrlCreateButton("Cancel", 256, 96, 65, 25)
GUICtrlCreateIcon("C:\Windows\System32\shell32.dll", -189, 24, 24, 32, 32)
GUICtrlCreateLabel("Then wait for it to spin up", 80, 80, 123, 17)
GUISetState(@SW_SHOW)
#EndRegion ### END Koda GUI section ###

	Local $nMsg,$bRet = True

	While True
		$nMsg = GUIGetMsg()
		Switch $nMsg
			Case $cbtnCancel,$GUI_EVENT_CLOSE
				$bRet = False
				ExitLoop
		EndSwitch
		If DriveStatus($sDrvLtr)='READY' Then
			ExitLoop
		EndIf
	WEnd

	GUIDelete($frmInsertDisc)
	Return $bRet
EndFunc

; Called when user clicks on "Rename folder" button. The button is actually a label with $SS_NOTIFY
Func _FSF_DlgRenameFolder($hItem,$sPathParent,$sCurName,$sExclFolders)
	WinSetState('[ACTIVE]','',@SW_DISABLE)
	Local $hCaller = WinGetHandle('[ACTIVE]')

	Local $iLeft,$iTop
	_FSF_GetLeftTopForCentringInParent(441,217,  $iLeft,  $iTop)

Local $frmRenameFolder = GUICreate("Rename a folder", 406, 217, $iLeft, $iTop, _
	BitXOR($GUI_SS_DEFAULT_GUI,$WS_MINIMIZEBOX),-1,$g_hFSF_TV)
	GUISetFont(8.5,400,0,'Tahoma',Default,5)
#Region ### START Koda GUI section ### Form=F:\AutoIt scripts\cFileSelectFolder\frmrenamefolder2.kxf MODIFIED
;~ Local $frmRenameFolder = GUICreate("Rename a folder", 406, 222, 203, 356, $WS_POPUP)
GUICtrlCreateLabel("Parent", 64, 8, 35, 17)
$g_cInpFSF_NewName = GUICtrlCreateInput("", 64, 144, 329, 21)
;~ Local $g_cInpFSF_NewName = GUICtrlCreateInput("", 64, 144, 329, 21)
Local $cInpParent = GUICtrlCreateInput("", 64, 32, 329, 21, $ES_READONLY)
GUICtrlCreateLabel("Current folder name", 64, 64, 96, 17)
Local $cInpCurName = GUICtrlCreateInput("", 64, 88, 329, 21, BitOR($GUI_SS_DEFAULT_INPUT,$ES_READONLY))
Local $cBtnCancel = GUICtrlCreateButton("Cancel", 336, 184, 57, 25)
Local $cBtnOK = GUICtrlCreateButton("OK", 264, 184, 57, 25, $BS_DEFPUSHBUTTON)
Local $cLblInfo = GUICtrlCreateLabel("", 64, 188, 188, 17)
GUICtrlSetFont(-1, 8, 800, 0, "MS Sans Serif")
GUICtrlSetColor(-1, 0xFF0000)
GUICtrlCreateIcon("C:\Windows\System32\shell32.dll", -4, 8, 8, 48, 48)
GUICtrlCreateLabel("New folder name", 64, 120, 84, 17)
GUISetState(@SW_SHOW)
#EndRegion ### END Koda GUI section ###

	GUICtrlSetData($cInpParent,$sPathParent)
	GUICtrlSetData($cInpCurName,$sCurName)
	If _FSF_GetPathForItem($hItem)='' Then
		GUICtrlSetData($cLblInfo,'This is part of Windows. It cannot be renamed')
		GUICtrlSetState($cBtnOK,$GUI_DISABLE)
		GUICtrlSetState($g_cInpFSF_NewName,$GUI_DISABLE)
	EndIf

	$g_sFSF_EditingMode = 'renaming'

	Local $iMsg,$sRet,$s
	While 1
		If $g_bFSF_InpChanged Then
			GUICtrlSetData($cLblInfo,'')
			$g_bFSF_InpChanged = False
		EndIf
		$iMsg = GUIGetMsg()
		Switch $iMsg
			Case $GUI_EVENT_CLOSE,$cBtnCancel
				$sRet = ''
				ExitLoop
			Case $cBtnOK
				$s = GUICtrlRead($g_cInpFSF_NewName)
				$s = StringStripWS($s,1+2)	; left,right
				Local $a10Excl = StringSplit($sExclFolders,'|',0+2)	; char,0-based

				If Not _FSF_IsFolderNameLegal($s,$cLblInfo) Then
					ContinueLoop
				EndIf
				If _FSF_IsInVectorBase0($a10Excl,$s) Then
					GUICtrlSetData($cLblInfo,'This is an excluded folder')
					ContinueLoop
				EndIf
				Local $sPath =  $sPathParent&'\'&$s
				If $s=$sCurName Then
					GUICtrlSetData($cLblInfo,'This is the same name')
					ContinueLoop
				EndIf
				; e.g. MProgs (F:)\ ... to F:\ ...
				$sPath = _FSF_GetPathForItem($sPath)
				If FileExists($sPath) Then
					If StringInStr(FileGetAttrib($sPath),'D') Then
						GUICtrlSetData($cLblInfo,'This folder already exists')
					Else
						GUICtrlSetData($cLblInfo,'A file with this name exists')
					EndIf
					ContinueLoop
				EndIf
				$sRet = $s
				ExitLoop
		EndSwitch
	WEnd
	GUIDelete($frmRenameFolder)
	$g_sFSF_EditingMode = ''
	WinSetState($hCaller,'',@SW_ENABLE)
	Return $sRet
EndFunc

; Called when user clicks on "Delete folder" button. The button is actually a label with $SS_NOTIFY
Func _FSF_DlgDeleteFolder($hItem,$sTree)
	WinSetState('[ACTIVE]','',@SW_DISABLE)
	Local $hCaller = WinGetHandle('[ACTIVE]')

Local $iLeft,$iTop
	_FSF_GetLeftTopForCentringInParent(397,175,  $iLeft,  $iTop)

Local $frmDelFolder = GUICreate("Delete folder or folders", 397, 175, $iLeft,$iTop, _
	BitXOR($GUI_SS_DEFAULT_GUI,$WS_MINIMIZEBOX),-1,$g_hFSF_TV)
GUISetFont(8.5,400,0,'Tahoma',Default,5)
#Region ### START Koda GUI section ### Form=f:\autoit scripts\cfileselectfolder\frmdelfolder2.kxf
;~ Local $frmDelFolder = GUICreate("Delete folder or folders", 397, 175, 210, 635)
GUICtrlCreateLabel("Which would you like to do:", 64, 56, 135, 17)
Local $cradRecycle = GUICtrlCreateRadio("Delete this folder (and any sub-folders) to the Recyle Bin", 64, 72, 289, 25)
GUICtrlSetState(-1, $GUI_CHECKED)
Local $cradPermanent = GUICtrlCreateRadio("Delete this folder (and any subfolders) permanently", 64, 96, 289, 25)
#forceref $cradPermanent
Local $cBtnCancel = GUICtrlCreateButton("Cancel", 320, 136, 57, 25)
Local $cBtnOK = GUICtrlCreateButton("OK", 248, 136, 57, 25)
Local $cInpFolder = GUICtrlCreateInput("", 64, 16, 321, 21, $ES_READONLY)
Local $cLblInfo = GUICtrlCreateLabel("", 64, 140, 180, 17)
GUICtrlSetColor(-1, 0xFF0000)
GUICtrlCreateIcon("C:\Windows\System32\shell32.dll", -4, 8, 8, 48, 48)
GUISetState(@SW_SHOW)
#EndRegion ### END Koda GUI section ###

	GUICtrlSetData($cInpFolder,$sTree)

	If _FSF_GetPathForItem($hItem)='' Then
		GUICtrlSetData($cLblInfo,'This is part of Windows. It cannot be deleted')
		GUICtrlSetState($cBtnOK,$GUI_DISABLE)
	EndIf

	Local $iMsg,$sRet
	While 1
		$iMsg = GUIGetMsg()
		Switch $iMsg
			Case $GUI_EVENT_CLOSE,$cBtnCancel
				$sRet = ''
				ExitLoop
			Case $cBtnOK
				If GUICtrlRead($cradRecycle)=$GUI_CHECKED Then
					$sRet = 'r'
				Else
					$sRet = 'd'
				EndIf
				ExitLoop
		EndSwitch
	WEnd
	GUIDelete($frmDelFolder)
	WinSetState($hCaller,'',@SW_ENABLE)
	Return $sRet
EndFunc

; Called when user clicks on "New folder" button. The button is actually a label with $SS_NOTIFY
; IF OK, returns folder name else returns ''
Func _FSF_DlgCreateNewFolder($hItemParent,$sPathParent,$sExclFolders)
	WinSetState('[ACTIVE]','',@SW_DISABLE)
	Local $hCaller = WinGetHandle('[ACTIVE]')

	Local $iLeft,$iTop
	_FSF_GetLeftTopForCentringInParent(406,166,  $iLeft, $iTop)

Local $frmCreateChildFolder = GUICreate("Create new  subfolder", 406, 166, $iLeft, $iTop, _
	BitXOR($GUI_SS_DEFAULT_GUI,$WS_MINIMIZEBOX),-1,$g_hFSF_TV)
	GUISetFont(8.5,400,0,'Tahoma',Default,5)
#Region ### START Koda GUI section ### Form=F:\AutoIt scripts\cFileSelectFolder\frmcreatechildfolder2.kxf MODIFIED
;~ Local $frmCreateChildFolder = GUICreate("Create new child folder", 406, 166, 202, 141, $WS_POPUP)
GUICtrlCreateLabel("Current folder:", 64, 8, 70, 17)
Local $cInpParent = GUICtrlCreateInput("", 64, 32, 329, 21, $ES_READONLY)
GUICtrlCreateLabel("New child folder:", 64, 64, 83, 17)
$g_cInpFSF_NewChild = GUICtrlCreateInput("", 64, 88, 329, 21)
;~ Local $cInpNewChild = GUICtrlCreateInput("", 64, 88, 329, 21)
Local $cBtnCancel = GUICtrlCreateButton("Cancel", 336, 128, 57, 25)
Local $cBtnOK = GUICtrlCreateButton("OK", 264, 128, 57, 25, $BS_DEFPUSHBUTTON)
Local $cLblInfo = GUICtrlCreateLabel("", 64, 132, 188, 17)
GUICtrlSetFont(-1, 8, 800, 0, "MS Sans Serif")
GUICtrlSetColor(-1, 0xFF0000)
GUICtrlCreateIcon("C:\Windows\System32\shell32.dll", -4, 8, 8, 48, 48)
GUISetState(@SW_SHOW)
#EndRegion ### END Koda GUI section ###

	GUICtrlSetData($cInpParent,$sPathParent)

	If _FSF_GetPathForItem($hItemParent)='' Then
		GUICtrlSetData($cLblInfo,'Folders cannot be created here')
		GUICtrlSetState($cBtnOK,$GUI_DISABLE)
		GUICtrlSetState($g_cInpFSF_NewChild,$GUI_DISABLE)
	EndIf

	$g_sFSF_EditingMode = 'creating'
	Local $iMsg,$sRet,$s
	While 1
		If $g_bFSF_InpChanged Then
			GUICtrlSetData($cLblInfo,'')
			$g_bFSF_InpChanged = False
		EndIf
		$iMsg = GUIGetMsg()
		Switch $iMsg
			Case $GUI_EVENT_CLOSE,$cBtnCancel
				$sRet = ''
				ExitLoop
			Case $cBtnOK
				$s = GUICtrlRead($g_cInpFSF_NewChild)
				$s = StringStripWS($s,1+2)	; left,right

				If Not _FSF_IsFolderNameLegal($s,$cLblInfo) Then
					ContinueLoop
				EndIf
				Local $a10Excl = StringSplit($sExclFolders,'|',0+2)	; char,0-based
				If _FSF_IsInVectorBase0($a10Excl,$s) Then
					GUICtrlSetData($cLblInfo,'This is an excluded folder')
					ContinueLoop
				EndIf
				; e.g. MProgs (F:)\ ... to F:\ ...
				Local $sPath = _FSF_GetPathForItem($hItemParent)&'\'&$s
				If FileExists($sPath) Then
					If StringInStr(FileGetAttrib($sPath),'D') Then
						GUICtrlSetData($cLblInfo,'This folder already exists')
					Else
						GUICtrlSetData($cLblInfo,'A file with this name exists')
					EndIf
					ContinueLoop
				EndIf
				$sRet = $s
				ExitLoop
		EndSwitch
	WEnd
	GUIDelete($frmCreateChildFolder)
	$g_sFSF_EditingMode = ''
	WinSetState($hCaller,'',@SW_ENABLE)
	Return $sRet
EndFunc

Func _FSF_IsFolderNameLegal($sName,$cLblInfo)
	If StringLen($sName)=0 Then
		GUICtrlSetData($cLblInfo,'A name is required')
		Return False
	EndIf

	Local $a10 = StringSplit($sName,$g_kFSF_sIllegalChars,0+2)	; char,0-based
	If UBound($a10)>1 Then
		GUICtrlSetData($cLblInfo,'Illegal character "'&StringMid($sName,StringLen($a10[0])+1,1)&'"')
		Return False
	EndIf

	If StringRight($sName,1)='.' Then
		GUICtrlSetData($cLblInfo,'Name cannot end with a period')
		Return False
	EndIf

	If StringRegExp($sName,'(?i)CON|PRN|AUX|NUL|COM[1-9]|LPT[1-9]') Then
		GUICtrlSetData($cLblInfo,'Illegal name')
		Return False
	EndIf
	Return True
EndFunc

; Ensures as many of the expanded items are visible, especially the parent
Func _FSF_EnsureTopOfExpansionIsVisible($hTreeView,$hParent)
	Local $hLastChild = _GUICtrlTreeView_GetLastChild($hTreeView,$hParent)
	If $hLastChild<>0 Then
		_GUICtrlTreeView_EnsureVisible($hTreeView, $hLastChild)
	EndIf
	_GUICtrlTreeView_EnsureVisible($hTreeView, $hParent)
EndFunc

; Returns $a1Ret[0] if $sPC not connected or no shares on the specified computer
Func _FSF_GetShareNamesOnPC($sPC)
Local $a10Ret[0]
	If $sPC=@ComputerName Then
		Local $a20ShareNamesPaths = _FSF_GetLocalShareNamesAndPaths()
		ReDim $a10Ret[UBound($a20ShareNamesPaths,1)]
		For $i = 0 To UBound($a20ShareNamesPaths,1)-1
			$a10Ret[$i] = $a20ShareNamesPaths[$i][0]
		Next
	Else
		Local $tResource = DllStructCreate($tagNETRESOURCE)
		Local $hEnum
		Local $iBufSize = 16384
		Local $tBuffer = DllStructCreate("byte[" & $iBufSize & "]")
		Local $pBuffer = DllStructGetPtr($tBuffer)
		Local $iCount = -1
		_FSF_cWinNet_OpenEnum($RESOURCE_REMEMBERED,$RESOURCETYPE_DISK,$RESOURCEUSAGE_CONNECTABLE,$tResource,$hEnum)

		_WinNet_EnumResource($hEnum,  $iCount,$pBuffer,  $iBufSize)
		If $iCount=1 Then
			_WinNet_CloseEnum($hEnum)
			Return $a10Ret
		EndIf

		Redim $a10Ret[$iCount]
		Local $tstruct
		Local $pEnum = $pBuffer
		Local $iqShares=0,$sRemote
		Local $iLen = 2+StringLen($sPC)
		For $i = 1 To $iCount
			$tstruct = DllStructCreate($tagNETRESOURCE,$pEnum)
			$sRemote = _FSF_PointerToStringW(DllStructGetData($tstruct,'RemoteName'))
			If StringLeft($sRemote,$iLen)='\\'&$sPC Then
				$a10Ret[$iqShares] = StringMid($sRemote,$iLen+2)
				$iqShares += 1
			EndIf
			$pEnum += 32
		Next
		ReDim $a10Ret[$iqShares]
		_WinNet_CloseEnum($hEnum)
	EndIf
	Return $a10Ret
EndFunc

Func _FSF_GetLocalShareNamesAndPaths()
Local $a20Ret[0][2]

	Local $wbemFlagReturnImmediately = 0x10
	Local	$wbemFlagForwardOnly = 0x20
	Local $strComputer = "localhost"
	Local $objWMIService = ObjGet("winmgmts:\\" & $strComputer & "\root\CIMV2")
	Local $colItems = $objWMIService.ExecQuery("SELECT Name,Path FROM Win32_Share WHERE Type = 0", "WQL", _
		$wbemFlagReturnImmediately + $wbemFlagForwardOnly)

	If IsObj($colItems) then
		ReDim $a20Ret[100][2]
		Local $iQ=0
			For $objItem In $colItems
			; if not print$
			If StringRight($objItem.Name,1)<>'$' Then
				$a20Ret[$iQ][0] =  $objItem.Name
				$a20Ret[$iQ][1] = $objItem.Path
				$iQ += 1
			EndIf
		Next
		ReDim $a20Ret[$iQ][2]
	Endif
	Return $a20Ret
EndFunc

; Returns [[drvltr,path], ...
; Returns $a1Ret[0][2] if $sPC not connected or no shares on the specified computer
Func _FSF_GetDriveMapping()
Local $a20Ret[0][2]
	Local $tResource = DllStructCreate($tagNETRESOURCE)
	Local $hEnum
	Local $iBufSize = 16384
	Local $pBuffer = _MemGlobalAlloc($iBufSize)
	Local $iCount = -1
	_FSF_cWinNet_OpenEnum($RESOURCE_CONNECTED,$RESOURCETYPE_DISK,$RESOURCEUSAGE_CONNECTABLE,$tResource,  $hEnum)
	_WinNet_EnumResource($hEnum,  $iCount,$pBuffer,  $iBufSize)
	If $iCount=1 Then
		_WinNet_CloseEnum($hEnum)
		Return $a20Ret
	EndIf

	Redim $a20Ret[$iCount][2]
	Local $tstruct
	Local $pEnum = $pBuffer
	Local $iqShares=0,$sRemote
	For $i = 1 To $iCount
		$tstruct = DllStructCreate($tagNETRESOURCE,$pEnum)
		$sRemote = _FSF_PointerToStringW(DllStructGetData($tstruct,'RemoteName'))
		$a20Ret[$iqShares][0] = _FSF_PointerToStringW(DllStructGetData($tstruct,'LocalName'))
		$a20Ret[$iqShares][1] = $sRemote
		$iqShares += 1
		$pEnum += 32
	Next
	ReDim $a20Ret[$iqShares][2]
	_WinNet_CloseEnum($hEnum)
	_MemGlobalFree($pBuffer)
	Return $a20Ret
EndFunc

; $iTimeOut: in seconds
Func _FSF_VarSizeDialog($title,$text,$iTimeOut)
	WinSetState('[ACTIVE]','',@SW_DISABLE)
	Local $hCaller = WinGetHandle('[ACTIVE]')

	Local $widDelta
	Local $fontNameEdt = 'Tahoma'
	Local $fontSize = 9
	Local $fontWeightEdt = 400

;~ 					_StringSize($sText[, $iSize[, $iWeight[, $iAttrib[, $sName[, $iWidth[, $hWnd]]]]]]
	Local $vec = _FSF_StringSize($text&@CRLF&@CRLF&@CRLF,$fontSize,$fontWeightEdt,0,$fontNameEdt,0,0)
	Local $widText = $vec[2]
	Local $htText = $vec[3]*1.05	; kludge
	Local $widEdt = $widText + 20 + 10	; kludge!
	Local $widBtns = (552-256) + 49
	If $widEdt<$widBtns Then $widEdt = $widBtns
	Local $iWidClient = $widEdt + 20
	Local $hwin = GUICreate('temp')
	Local $lbl = GUICtrlCreateLabel('tmep',1,1)

	$vec = _FSF_GUICtrlGetFont($lbl)
	Local $fontNameTit = $vec[3]
	Local $fontSizeTit = $vec[0]
	GUIDelete($hwin)

	$vec = _FSF_StringSize($title&'',$fontSizeTit,700,0,$fontNameTit,0,0); font size is a guess!  bolded

	Local $widTitle = $vec[2]
	If $widTitle+80>$iWidClient Then
		Local $deltaTit = $widTitle+80 - $iWidClient
		$widEdt += $deltaTit
		$iWidClient += $deltaTit
	EndIf

	Local $leftOK = 10 + $widEdt - 49
	Local $widWin = $iWidClient + 10

	If $widWin>@DesktopWidth Then
		$widDelta = $widWin - @DesktopWidth
		$widEdt -= $widDelta
		$iWidClient -= $widDelta
		$leftOK -= $widDelta
	EndIf

	Local $htEdt = $htText - 20
	If $htEdt<126 Then $htEdt =  126

	Local $iHtClient = 10 + $htEdt + 20 + 25
	Local $topBtn = 10 + $htEdt + 10
	Local $htWin = $iHtClient + 10

	If $htWin>@DesktopHeight-50 Then
		Local $htDelta =  $htWin - (@DesktopHeight-50)
		$htEdt -= $htDelta
		$iHtClient -= $htDelta
		$topBtn -= $htDelta
	EndIf

	Local $hTimer=0
	If $iTimeOut<>0 Then
		$hTimer = TimerInit
	EndIf

	Local $iLeftClient,$iTopClient
	_FSF_GetLeftTopForCentringInParent($iWidClient,$iHtClient+20,  $iLeftClient,  $iTopClient)

	Local $Form1 = GUICreate($title, $iWidClient, $iHtClient+20,$iLeftClient,$iTopClient,BitOR($WS_SIZEBOX,$WS_MAXIMIZEBOX))
	Local $edt = GUICtrlCreateEdit("", 8, 8, $widEdt, $htEdt,$ES_READONLY)
	GUICtrlSetFont(-1,$fontSize,$fontWeightEdt,0,$fontNameEdt)
	GUICtrlSetBkColor(-1, 0xFFFBF0)	; cream
	Local $btnOK = GUICtrlCreateButton("OK", $leftOK, $topBtn, 49, 25, $BS_DEFPUSHBUTTON)
	GUISetState(@SW_SHOW)

	GUICtrlSetData($edt,$text)

	Local $oldWinHandle = WinGetHandle('[ACTIVE]')
	GUISwitch($Form1)

	Local $nMsg,$bEndit=False
	While 1
		If $hTimer<>0 And TimerDiff($hTimer)>=$iTimeOut*1000 Then
			ExitLoop
		EndIf
		$nMsg = GUIGetMsg()
		Switch $nMsg
			Case $GUI_EVENT_CLOSE,$btnOK
				$bEndit = true
				ExitLoop
		EndSwitch
	WEnd
	GUISwitch($oldWinHandle)
	GUIDelete($Form1)
	WinSetState($hCaller,'',@SW_ENABLE)
	Return $bEndit
EndFunc

; #FUNCTION# =========================================================================================================
; Name...........: _FSF_StringSize
; Description ...: Returns size of rectangle required to display string - maximum permitted width can be chosen
; Syntax ........: _FSF_StringSize($sText[, $iSize[, $iWeight[, $iAttrib[, $sName[, $iWidth[, $hWnd]]]]]])
; Parameters ....: $sText   - String to display
;                  $iSize   - [optional] Font size in points - (default = 8.5)
;                  $iWeight - [optional] Font weight - (default = 400 = normal)
;                  $iAttrib - [optional] Font attribute (0-Normal (default), 2-Italic, 4-Underline, 8 Strike)
;                             + 1 if tabs are to be expanded before sizing
;                  $sName   - [optional] Font name - (default = Tahoma)
;                  $iWidth  - [optional] Max width for rectangle - (default = 0 => width of original string)
;                  $hWnd    - [optional] GUI in which string will be displayed - (default 0 => normally not required)
; Requirement(s) : v3.2.12.1 or higher
; Return values .: Success - Returns 4-element array: ($iWidth set // $iWidth not set)
;                  |$array[0] = String reformatted with additonal @CRLF // Original string
;                  |$array[1] = Height of single line in selected font // idem
;                  |$array[2] = Width of rectangle required for reformatted // original string
;                  |$array[3] = Height of rectangle required for reformatted // original string
;                  Failure - Returns 0 and sets @error:
;                  |1 - Incorrect parameter type (@extended = parameter index)
;                  |2 - DLL call error - extended set as follows:
;                       |1 - GetDC failure
;                       |2 - SendMessage failure
;                       |3 - GetDeviceCaps failure
;                       |4 - CreateFont failure
;                       |5 - SelectObject failure
;                       |6 - GetTextExtentPoint32 failure
;                  |3 - Font too large for chosen max width - a word will not fit
; Author ........: Melba23 - thanks to trancexx for the default DC code
; Modified ......:
; Remarks .......: The use of the $hWnd parameter is not normally necessary - it is only required if the UDF does not
;                   return correct dimensions without it.
; Related .......:
; Link ..........:
; Example .......: Yes
;=====================================================================================================================
Func _FSF_StringSize($sText, $iSize = 8.5, $iWeight = 400, $iAttrib = 0, $sName = "", $iMaxWidth = 0, $hWnd = 0)

	; Set parameters passed as Default
	If $iSize = Default Then $iSize = 8.5
	If $iWeight = Default Then $iWeight = 400
	If $iAttrib = Default Then $iAttrib = 0
	If $sName = "" Or $sName = Default Then	$sName = _FSF_StringSize_DefaultFontName()

	; Check parameters are correct type
	If Not IsString($sText) Then Return SetError(1, 1, 0)
	If Not IsNumber($iSize) Then Return SetError(1, 2, 0)
	If Not IsInt($iWeight) Then Return SetError(1, 3, 0)
	If Not IsInt($iAttrib) Then Return SetError(1, 4, 0)
	If Not IsString($sName) Then Return SetError(1, 5, 0)
	If Not IsNumber($iMaxWidth) Then Return SetError(1, 6, 0)
	If Not IsHwnd($hWnd) And $hWnd <> 0 Then Return SetError(1, 7, 0)
	Local $aRet, $hDC, $hFont, $hLabel = 0, $hLabel_Handle

	; Check for tab expansion flag
	Local $iExpTab = BitAnd($iAttrib, 1)
	; Remove possible tab expansion flag from font attribute value
	$iAttrib = BitAnd($iAttrib, BitNot(1))

	; If GUI handle was passed
	If IsHWnd($hWnd) Then
		; Create label outside GUI borders
		$hLabel = GUICtrlCreateLabel("", -10, -10, 10, 10)
		$hLabel_Handle = GUICtrlGetHandle(-1)
		GUICtrlSetFont(-1, $iSize, $iWeight, $iAttrib, $sName)
		; Create DC
		$aRet = DllCall("user32.dll", "handle", "GetDC", "hwnd", $hLabel_Handle)
		If @error Or $aRet[0] = 0 Then
			GUICtrlDelete($hLabel)
			Return SetError(2, 1, 0)
		EndIf
		$hDC = $aRet[0]
		$aRet = DllCall("user32.dll", "lparam", "SendMessage", "hwnd", $hLabel_Handle, "int", 0x0031, "wparam", 0, "lparam", 0) ; $WM_GetFont
		If @error Or $aRet[0] = 0 Then
			GUICtrlDelete($hLabel)
			Return SetError(2, _FSF_StringSize_Error_Close(2, $hDC), 0)
		EndIf
		$hFont = $aRet[0]
	Else
		; Get default DC
		$aRet = DllCall("user32.dll", "handle", "GetDC", "hwnd", $hWnd)
		If @error Or $aRet[0] = 0 Then Return SetError(2, 1, 0)
		$hDC = $aRet[0]
		; Create required font
		$aRet = DllCall("gdi32.dll", "int", "GetDeviceCaps", "handle", $hDC, "int", 90) ; $LOGPIXELSY
		If @error Or $aRet[0] = 0 Then Return SetError(2, _FSF_StringSize_Error_Close(3, $hDC), 0)
		Local $iInfo = $aRet[0]
		$aRet = DllCall("gdi32.dll", "handle", "CreateFontW", "int", -$iInfo * $iSize / 72, "int", 0, "int", 0, "int", 0, _
			"int", $iWeight, "dword", BitAND($iAttrib, 2), "dword", BitAND($iAttrib, 4), "dword", BitAND($iAttrib, 8), "dword", 0, "dword", 0, _
			"dword", 0, "dword", 5, "dword", 0, "wstr", $sName)
		If @error Or $aRet[0] = 0 Then Return SetError(2, _FSF_StringSize_Error_Close(4, $hDC), 0)
		$hFont = $aRet[0]
	EndIf

	; Select font and store previous font
	$aRet = DllCall("gdi32.dll", "handle", "SelectObject", "handle", $hDC, "handle", $hFont)
	If @error Or $aRet[0] = 0 Then Return SetError(2, _FSF_StringSize_Error_Close(5, $hDC, $hFont, $hLabel), 0)
	Local $hPrevFont = $aRet[0]

	; Declare variables
    Local $avSize_Info[4], $iLine_Length, $iLine_Height = 0, $iLine_Count = 0, $iLine_Width = 0, $iWrap_Count, $iLast_Word, $sTest_Line
	; Declare and fill Size structure
	Local $tSize = DllStructCreate("int X;int Y")
	DllStructSetData($tSize, "X", 0)
	DllStructSetData($tSize, "Y", 0)

	; Ensure EoL is @CRLF and break text into lines
	$sText = StringRegExpReplace($sText, "((?<!\x0d)\x0a|\x0d(?!\x0a))", @CRLF)
	Local $asLines = StringSplit($sText, @CRLF, 1)

	; For each line
	For $i = 1 To $asLines[0]
		; Expand tabs if required
		If $iExpTab Then
			$asLines[$i] = StringReplace($asLines[$i], @TAB, " XXXXXXXX")
		EndIf
		; Size line
		$iLine_Length = StringLen($asLines[$i])
		DllCall("gdi32.dll", "bool", "GetTextExtentPoint32W", "handle", $hDC, "wstr", $asLines[$i], "int", $iLine_Length, "ptr", DllStructGetPtr($tSize))
		If @error Then Return SetError(2, _FSF_StringSize_Error_Close(6, $hDC, $hFont, $hLabel), 0)
		If DllStructGetData($tSize, "X") > $iLine_Width Then $iLine_Width = DllStructGetData($tSize, "X")
		If DllStructGetData($tSize, "Y") > $iLine_Height Then $iLine_Height = DllStructGetData($tSize, "Y")
	Next

	; Check if $iMaxWidth has been both set and exceeded
	If $iMaxWidth <> 0 And $iLine_Width > $iMaxWidth Then ; Wrapping required
		; For each Line
		For $j = 1 To $asLines[0]
			; Size line unwrapped
			$iLine_Length = StringLen($asLines[$j])
			DllCall("gdi32.dll", "bool", "GetTextExtentPoint32W", "handle", $hDC, "wstr", $asLines[$j], "int", $iLine_Length, "ptr", DllStructGetPtr($tSize))
			If @error Then Return SetError(2, _FSF_StringSize_Error_Close(6, $hDC, $hFont, $hLabel), 0)
			; Check wrap status
			If DllStructGetData($tSize, "X") < $iMaxWidth - 4 Then
				; No wrap needed so count line and store
				$iLine_Count += 1
				$avSize_Info[0] &= $asLines[$j] & @CRLF
			Else
				; Wrap needed so zero counter for wrapped lines
				$iWrap_Count = 0
				; Build line to max width
				While 1
					; Zero line width
					$iLine_Width = 0
					; Initialise pointer for end of word
					$iLast_Word = 0
					; Add characters until EOL or maximum width reached
					For $i = 1 To StringLen($asLines[$j])
						; Is this just past a word ending?
						If StringMid($asLines[$j], $i, 1) = " " Then $iLast_Word = $i - 1
						; Increase line by one character
						$sTest_Line = StringMid($asLines[$j], 1, $i)
						; Get line length
						$iLine_Length = StringLen($sTest_Line)
						DllCall("gdi32.dll", "bool", "GetTextExtentPoint32W", "handle", $hDC, "wstr", $sTest_Line, "int", $iLine_Length, "ptr", DllStructGetPtr($tSize))
						If @error Then Return SetError(2, _FSF_StringSize_Error_Close(6, $hDC, $hFont, $hLabel), 0)
						$iLine_Width = DllStructGetData($tSize, "X")
						; If too long exit the loop
						If $iLine_Width >= $iMaxWidth - 4 Then ExitLoop
					Next
					; End of the line of text?
					If $i > StringLen($asLines[$j]) Then
						; Yes, so add final line to count
						$iWrap_Count += 1
						; Store line
						$avSize_Info[0] &= $sTest_Line & @CRLF
						ExitLoop
					Else
						; No, but add line just completed to count
						$iWrap_Count += 1
						; Check at least 1 word completed or return error
						If $iLast_Word = 0 Then Return SetError(3, _FSF_StringSize_Error_Close(0, $hDC, $hFont, $hLabel), 0)
						; Store line up to end of last word
						$avSize_Info[0] &= StringLeft($sTest_Line, $iLast_Word) & @CRLF
						; Strip string to point reached
						$asLines[$j] = StringTrimLeft($asLines[$j], $iLast_Word)
						; Trim leading whitespace
						$asLines[$j] = StringStripWS($asLines[$j], 1)
						; Repeat with remaining characters in line
					EndIf
				WEnd
				; Add the number of wrapped lines to the count
				$iLine_Count += $iWrap_Count
			EndIf
		Next
		; Reset any tab expansions
		If $iExpTab Then
			$avSize_Info[0] = StringRegExpReplace($avSize_Info[0], "\x20?XXXXXXXX", @TAB)
		EndIf
		; Complete return array
		$avSize_Info[1] = $iLine_Height
		$avSize_Info[2] = $iMaxWidth
		; Convert lines to pixels and add drop margin
		$avSize_Info[3] = ($iLine_Count * $iLine_Height) + 4
	Else ; No wrapping required
		; Create return array (add drop margin to height)
		Local $avSize_Info[4] = [$sText, $iLine_Height, $iLine_Width, ($asLines[0] * $iLine_Height) + 4]
	EndIf

	; Clear up
    DllCall("gdi32.dll", "handle", "SelectObject", "handle", $hDC, "handle", $hPrevFont)
	DllCall("gdi32.dll", "bool", "DeleteObject", "handle", $hFont)
	DllCall("user32.dll", "int", "ReleaseDC", "hwnd", 0, "handle", $hDC)
	If $hLabel Then GUICtrlDelete($hLabel)
	Return $avSize_Info
EndFunc ;==>_FSF_StringSize

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: _FSF_StringSize_Error_Close
; Description ...: Releases DC and deleted font object if required after error
; Syntax ........: _FSF_StringSize_Error_Close ($iExtCode, $hDC, $hGUI)
; Parameters ....: $iExtCode   - code to return
;                  $hDC, $hGUI - handles as set in _FSF_StringSize function
; Return value ..: $iExtCode as passed
; Author ........: Melba23
; Modified.......:
; Remarks .......: This function is used internally by _FSF_StringSize
; ===============================================================================================================================
Func _FSF_StringSize_Error_Close($iExtCode, $hDC = 0, $hFont = 0, $hLabel = 0)
	If $hFont <> 0 Then DllCall("gdi32.dll", "bool", "DeleteObject", "handle", $hFont)
	If $hDC <> 0 Then DllCall("user32.dll", "int", "ReleaseDC", "hwnd", 0, "handle", $hDC)
	If $hLabel Then GUICtrlDelete($hLabel)
	Return $iExtCode
EndFunc ;=>_FSF_StringSize_Error_Close

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: _FSF_StringSize_DefaultFontName
; Description ...: Determines Windows default font
; Syntax ........: _FSF_StringSize_DefaultFontName()
; Parameters ....: None
; Return values .: Success - Returns name of system default font
;                  Failure - Returns "Tahoma"
; Author ........: Melba23, based on some original code by Larrydalooza
; Modified.......:
; Remarks .......: This function is used internally by _FSF_StringSize
; ===============================================================================================================================
Func _FSF_StringSize_DefaultFontName()

	; Get default system font data
	Local $tNONCLIENTMETRICS = DllStructCreate("uint;int;int;int;int;int;byte[60];int;int;byte[60];int;int;byte[60];byte[60];byte[60]")
	DLLStructSetData($tNONCLIENTMETRICS, 1, DllStructGetSize($tNONCLIENTMETRICS))
	DLLCall("user32.dll", "int", "SystemParametersInfo", "int", 41, "int", DllStructGetSize($tNONCLIENTMETRICS), "ptr", DllStructGetPtr($tNONCLIENTMETRICS), "int", 0)
	Local $tLOGFONT = DllStructCreate("long;long;long;long;long;byte;byte;byte;byte;byte;byte;byte;byte;char[32]", DLLStructGetPtr($tNONCLIENTMETRICS, 13))
	If IsString(DllStructGetData($tLOGFONT, 14)) Then
		Return DllStructGetData($tLOGFONT, 14)
	Else
		Return "Tahoma"
	EndIf
EndFunc ;=>_FSF_StringSize_DefaultFontName

; #FUNCTION# ====================================================================================================================
; Name...........: _FSF_GUICtrlGetFont
; Description ...: Gets the font of a GUI Control
; Syntax.........: _FSF_GUICtrlGetFont( [$hWnd] )
; Parameters ....: $hWnd        - [optional] ControlID or Handle of the control.
;                                 Default is last created GUICtrl... (-1)
;
; Return values .: Success      - Array[5] with options of font:
;                                 [0] - FontSize (~ approximation)
;                                 [1] - font weight (400 = normal).
;                                 [2] - italic:2 underlined:4 strike:8 char format (styles added together,
;                                       2+4 = italic and underlined).
;                                 [3] - The name of the font in use.
;                                 [4] - font quality to select (PROOF_QUALITY=2 is default in AutoIt).
;
;                  Failure      - Array[5] with empty fields, @error set to nonzero
;
; Author ........: KaFu, Prog@ndy
;
; Comments.......: The FontSize returned is an approximation of the actual FontSize used for the control.
;                  The height of the font returned by GetObject is the height of the font's character cell or
;                  character in logical units.
;                  The character height value (also known as the em height) is the character cell height value
;                  minus the internal-leading value.
;                  The font mapper interprets the value specified in lfHeight. The result returned by the font
;                  mapper is not easily reversible
;                  The FontSize calculated below is an approximation of the actual size used for the analyzed
;                  control, qualified enough to use in another call to the font mapper resulting in the same
;                  font size as the the original font.
; MSDN.. ........: Windows font Mapping: http://msdn.microsoft.com/en-us/library/ms969909(loband).aspx
; ===============================================================================================================================
; http://www.autoitscript.com/forum/topic/124526-guictrlgetfont
Func _FSF_GUICtrlGetFont($hWnd = -1)
    Local $aReturn[5], $hObjOrg
    If Not IsHWnd($hWnd) Then $hWnd = GUICtrlGetHandle($hWnd)
    If Not IsHWnd($hWnd) Then Return SetError(1, 0, $aReturn)
    Local $hFont = _SendMessage($hWnd, $WM_GETFONT)
    If Not $hFont Then Return SetError(2, 0, $aReturn)
    Local $hDC = _WinAPI_GetDC($hWnd)
    $hObjOrg = _WinAPI_SelectObject($hDC, $hFont)
    Local $tFONT = DllStructCreate($tagLOGFONT)
    Local $aRet = DllCall('gdi32.dll', 'int', 'GetObjectW', 'ptr', $hFont, 'int', DllStructGetSize($tFONT), 'ptr', _
		DllStructGetPtr($tFONT))
    If @error Or $aRet[0] = 0 Then
        _WinAPI_SelectObject($hDC, $hObjOrg)
        _WinAPI_ReleaseDC($hWnd, $hDC)
        Return SetError(3, 0, $aReturn)
    EndIf

    ; Need to extract FontFacename separately  => DllStructGetData($tFONT, 'FaceName') is only valid if
	; FontFacename has been set explicitly!
    $aRet = DllCall("gdi32.dll", "int", "GetTextFaceW", "handle", $hDC, "int", 0, "ptr", 0)
    Local $nCount = $aRet[0]
    Local $tBuffer = DllStructCreate("wchar[" & $aRet[0] & "]")
    Local $pBuffer = DllStructGetPtr($tBuffer)
    $aRet = DllCall("Gdi32.dll", "int", "GetTextFaceW", "handle", $hDC, "int", $nCount, "ptr", $pBuffer)
    If @error Then
        _WinAPI_SelectObject($hDC, $hObjOrg)
        _WinAPI_ReleaseDC($hWnd, $hDC)
        Return SetError(4, 0, $aReturn)
    EndIf
    $aReturn[3] = DllStructGetData($tBuffer, 1) ; FontFacename
    $aReturn[0] = Round((-1 * DllStructGetData($tFONT,'Height'))* 72/_WinAPI_GetDeviceCaps($hDC,90),1) ; $LOGPIXELSY = 90 => DPI aware
    _WinAPI_SelectObject($hDC, $hObjOrg)
    _WinAPI_ReleaseDC($hWnd, $hDC)
    $aReturn[1] = DllStructGetData($tFONT, 'Weight')
    $aReturn[2] = 2 * (True = DllStructGetData($tFONT, 'Italic')) + 4 * (True = DllStructGetData($tFONT, 'Underline')) + 8 * (True = DllStructGetData($tFONT, 'StrikeOut'))
    $aReturn[4] = DllStructGetData($tFONT, 'Quality')
    Return $aReturn
EndFunc   ;==>_FSF_GUICtrlGetFont

Func _FSF_IsInVectorBase0($a10,$vVal)
	Local $bRet=False
	For $i = 0 To UBound($a10)-1
		If $a10[$i]=$vVal Then
			$bRet = True
			ExitLoop
		EndIf
	Next
	Return $bRet
EndFunc

Func _FSF_GetLeftTopForCentringInParent($iChildWid,$iChildHt, ByRef $iChildLeft, ByRef $iChildTop)
	Local Enum $eLeft,$eTop,$eWid,$eHt
	Local $a10Parent = WinGetPos($g_hFSF_Win)
	$iChildLeft = $a10Parent[$eLeft] + ($a10Parent[$eWid]/2) - (($iChildWid+6)/2)	; 6 = frame
	$iChildTop = $a10Parent[$eTop] + ($a10Parent[$eHt]/2) - (($iChildHt+25)/2)			; 25 = frame + title bar
EndFunc


; #FUNCTION# =========================================================================================================
; Name...........: _FSF_WM_NOTIFY
; Description ...: Windows message handler for WM_NOTIFY - reacts to doubleclick and item expansion on TreeView
; Syntax.........: _FSF_WM_NOTIFY($hWnd, $iMsg, $wParam, $lParam)
; Requirement(s).: v3.3 +
; Return values..: None
; Author ........: Melba23 - thanks to guinness for help with the #*@#*%# Struct !!!!
; Modified ......:
; Remarks .......: If a WM_NOTIFY handler already registered, then call this function from within that handler
; Example........: Yes
;=====================================================================================================================
Func _FSF_WM_NOTIFY($hWnd, $iMsg, $wParam, $lParam)
	#forceref $hWnd, $iMsg, $wParam

	; Create NMTREEVIEW structure
	Local $tStruct = DllStructCreate("struct;hwnd hWndFrom;uint_ptr IDFrom;INT Code;endstruct;" & _
			"uint Action;struct;uint OldMask;handle OldhItem;uint OldState;uint OldStateMask;" & _
			"ptr OldText;int OldTextMax;int OldImage;int OldSelectedImage;int OldChildren;lparam OldParam;endstruct;" & _
			"struct;uint NewMask;handle NewhItem;uint NewState;uint NewStateMask;" & _
			"ptr NewText;int NewTextMax;int NewImage;int NewSelectedImage;int NewChildren;lparam NewParam;endstruct;" & _
			"struct;long PointX;long PointY;endstruct", $lParam)
	Local $hWndFrom = DllStructGetData($tStruct, "hWndFrom")
	Local $hItem = DllStructGetData($tStruct, "NewhItem")
	Local $iCode = DllStructGetData($tStruct, "Code")

	If $hWndFrom = $g_hFSF_TV Then
		; Check action
		Switch $iCode
;~ 			Case -2 ; $NM_CLICK
				; Fire the dummy control
;~ 				GUICtrlSendToDummy($g_cFSF_Click_Dummy)
			Case -3 ; $NM_DBLCLK 0xFFFFFFFD
				; Fire the dummy control
				GUICtrlSendToDummy($g_cFSF_DblClk_Dummy)
				; Set flag
			Case $TVN_ITEMEXPANDEDW, $TVN_ITEMEXPANDEDA
				; Fire the dummy control if expanding
				If DllStructGetData($tStruct, "Action") = 2 Then
					GUICtrlSendToDummy($g_cFSF_Expand_Dummy, $hItem)
				EndIf
			Case $TVN_SELCHANGEDA, $TVN_SELCHANGEDW
				; Fire the dummy control
				GUICtrlSendToDummy($g_cFSF_Select_Dummy, $hItem)
		EndSwitch
	EndIf
EndFunc   ;==>_CFF_WM_NOTIFY_Handler

; Prevent window from being smalled too much
Func _FSF_WM_GETMINMAXINFO($hWnd, $iMsg, $wParam, $lParam) ; Original idea by Zedna & updated by guinness.
#forceref $iMsg, $wParam
; Ref: https://www.autoitscript.com/forum/topic/149027-inssue-setting-max-height-on-resize/
; MINMAXINFO Struct >> http://msdn.microsoft.com/en-us/library/windows/desktop/ms632605(v=vs.85).aspx
; It uses a $tagPOINT structure which is long X; long Y;
Local Const $tagMINMAXINFO = 'struct; long ReservedX;long ReservedY;long MaxSizeX;long MaxSizeY;'& _
	'long MaxPositionX;long MaxPositionY;long MinTrackSizeX;long MinTrackSizeY;'& _
	'long MaxTrackSizeX;long MaxTrackSizeY; endstruct'
	If $hWnd=$g_hFSF_Win Then
		Local $tMINMAXINFO = DllStructCreate($tagMINMAXINFO, $lParam)
		DllStructSetData($tMINMAXINFO, 'MinTrackSizeX', $g_iFSF_GuiMinSizeX)
		DllStructSetData($tMINMAXINFO, 'MinTrackSizeY', $g_iFSF_GuiMinSizeY)
		DllStructSetData($tMINMAXINFO, 'MaxTrackSizeX', $g_iFSF_GuiMaxSizeX)
		DllStructSetData($tMINMAXINFO, 'MaxTrackSizeY', $g_iFSF_GuiMaxSizeY)
	EndIf
EndFunc   ;==>WM_GETMINMAXINFO

; #FUNCTION# ====================================================================================================================
; Name ..........: _cWinNet_OpenEnum
; Description ...: Starts an enumeration of network resources or existing connections
; Syntax ........: _cWinNet_OpenEnum($iScope, $iType, $iUsage, $tResource, ByRef $hEnum)
; Parameters ....: +$iScope
;                  | Scope of enumeration
;                                    $RESOURCE_CONNECTED (1) Enmuerate all currently connected resources.
;                  + Function ignores $iUsage
;                                     $RESOURCE_CONTEXT (5) Enumerate only resources in the network context of the
;                  + caller. Specify this value for a Network Neighborhood view. Function ignores $iUsage
;                                     $RESOURCE_GLOBALNET (2) Enumerate all resources on the network
;                                     $RESOURCE_REMEMBERED (3) Enumerate all remembered (persistent) connections.
;                  + Function ignores $iUsage
;                  -$iType
;                  |Resource types to be enumerated
;                                     $RESOURCETYPE_ANY (0) All resources
;                                     $RESOURCETYPE_DISK (1) All disk resources
;                                     $RESOURCETYPE_PRINT (2)All print resources
;                  -$iUsage
;                  |Resource usage type to be enumerated. BitOR to combine.
;                  + Ignored unless $iScope=$RESOURCE_GLOBALNET
;                                     0 All resources
;                                     $RESOURCEUSAGE_CONNECTABLE (1) All connectable resources
;                                     $RESOURCEUSAGE_CONTAINER (2) All container resources
;                                     $RESOURCEUSAGE_ATTACHED (16) Forces function to fail if the user is not authenticated.
;                  + The function fails even if the network allows enumeration without authentication
;                                     RESOURCEUSAGE_ALL (19) Same as
;                  + BitOR($RESOURCEUSAGE_CONNECTABLE,$RESOURCEUSAGE_CONTAINER,$RESOURCEUSAGE_ATTACHED)
;                  -$tResource
;                  |$tagNETRESOURCE structure that specifies the container to enumerate
;                                         Unless $iScope is $RESOURCE_GLOBALNET, this must be 0.
;                                         If 0, the root of the network is assumed.
;                  -$hEnum
;                  |If $hEnum is 0, on return, a handle that can be used in calls to <a href="_WinNet_EnumResource.htm">_WinNet_EnumResource()</a>
; Return values:   Success: True
;+
;                  Failure: 0 and sets @error:
;                  -     6
;                  |The handle specified by $hEnum is invalid
;                  -    87
;                  |Either $iScope or $iType is invalid or combination of parameters is invalid
;                  -   487
;                  |A remote network resource name in the structure that $hEnum points to resolves
;                  + to an invalid network address
;                  -  1204
;                  |The specified network provider name is invalid
;                  -  1207
;                  |$hEnum does not point to a container
;                  -  1208
;                  |Extended error: call _WinNet_GetLastError for the desciption
;                  -  1222
;                  |The network is not available
;                  - other
;                  |See APIErrorsConstants.au3
; Author ........: c.haslam
; Modified ......:
; Remarks .......: If $tResource is a $tagResource struct, $iUsage must be $RESOURCEUSAGE_CONTAINER
;+
;                  Is more general than _WinNet_OpenEnum and calls Unicode version, not ASNI
;+
; To enumerate all network resources, start by calling _WinNet_OpenEnum with $tResoruce set to 0,
;+ then use the returned (ByRef) handle to enumerate resources. If one of the resources in the $tRESOURCE
;+ buffer is a container resource, call WinNet_OpenEnum to open the resource for further enumeration.
;+ See the example below.
; Related .......: $tagNETRESOURCE, _WinNet_CloseEnum, _WinNet_EnumResource
; Link ..........: https://docs.microsoft.com/en-us/windows/desktop/api/winnetwk/nf-winnetwk-wnetopenenuma
; Example .......: No
; ===============================================================================================================================
Func _FSF_cWinNet_OpenEnum($iScope, $iType, $iUsage, $tResource, ByRef $hEnum)

	Local $aResult = DllCall("mpr.dll", "dword", "WNetOpenEnumW", "dword", $iScope, "dword", $iType, "dword", _
		$iUsage, "struct*", $tResource, "handle*", 0)
	If $aResult[0]=0 Then
		$hEnum = $aResult[5]
		Return True
	Else
		Return SetError($aResult[0], 0,False)
	EndIf
EndFunc

Func _FSF_PointerToStringW($ptr)
    Return DllStructGetData(DllStructCreate("wchar[" & _WinAPI_StringLenW($ptr) & "]", $ptr), 1)
EndFunc   ;==>_PointerToStringW

; Ref: https://www.autoitscript.com/forum/topic/54909-detecting-when-guictrlinput-change-it-value/
Func _FSF_WM_COMMAND($hWnd, $iMsg, $iwParam, $ilParam)
#forceref $hWnd,$iMsg
    Local $hWndFrom, $iCode, $hWndEdit
	Switch $g_sFSF_EditingMode
	Case 'creating'
		If IsHWnd($g_cInpFSF_NewChild) Then
			$hWndEdit = $g_cInpFSF_NewChild
		Else
			$hWndEdit = GUICtrlGetHandle($g_cInpFSF_NewChild)
		EndIf
	Case 'renaming'
		If IsHWnd($g_cInpFSF_NewName) Then
			$hWndEdit = $g_cInpFSF_NewName
		Else
			$hWndEdit = GUICtrlGetHandle($g_cInpFSF_NewName)
		EndIf
	EndSwitch
    $hWndFrom = $ilParam
;~     $iIDFrom = _WinAPI_LoWord ($iwParam)
    $iCode = _WinAPI_HiWord ($iwParam)
    Switch $hWndFrom
        Case $hWndEdit
            Switch $iCode
				; Sent when the user has taken an action that may have altered text in an edit control
                Case $EN_CHANGE
					$g_bFSF_InpChanged = True
                    ; no return value
			EndSwitch
	EndSwitch
EndFunc

; #FUNCTION# =========================================================================================================
; Name...........: _FSF_RegMsg
; Description ...: Registers WM_NOTIFY for TreeView doubleclick and item expansion and _FSF_WM_COMMAND for list focus
; Syntax.........: _CFF_RegMsg([$bNOTIFY = True[, $bCOMMAND = True]])
; Parameters.....: $bNOTIFY  - True (default) = Register _FSF_WM_NOTIFY handler function
;                  $bCOMMAND - True (default) = Register _FSF_WM_COMMAND handler function
; Requirement(s).: v3.3 +
; Return values .: 0 - No handlers registered
;                  1 - WM_NOTIFY handler registered
;                  2 - WM_COMMAND handler registered
;                  3 - Both handlers registerd
; Author ........: Melba23
; Modified ......: c.haslam
; Remarks .......: If a script has existing WM_NOTIFY or _WM_COMMAND handlers then do not register the relevant
;                  handler using this function but call it directly from within the existing message handler
; Example........: Yes
;=====================================================================================================================
Func _FSF_RegMsg($bNOTIFY=True,$bCOMMAND=True)

	; Register required messages
	Local $iRet = 0
	If $bNOTIFY Then
		$iRet = GUIRegisterMsg(0x004E, "_FSF_WM_NOTIFY") ; $WM_NOTIFY
	EndIf

	If $bCOMMAND Then
		$iRet += 2 * GUIRegisterMsg($WM_COMMAND, "_FSF_WM_COMMAND")
	EndIf
	Return $iRet
EndFunc   ;==>_CFF_RegMsg